/**
 * @file
 * @brief implementation of the channel group class
 * @author Miroslav Kubelik
 * @date 2007
 */

//STDC++ and STL Headers
#include <string>
#include <vector>
#include <iostream>
#include <stdexcept>

//C and local headers
#include "Changrp.h"
#include "GIface.h"
#include "Utils.h"

using namespace std;

//---------------------------------------------------------------------------
/**
 * copy constructor, nedded if use "dynamic memory" structures
 * in class members (e.g. vector of pointers, arrays of pointers)
 */
Changrp::Changrp(const Changrp& copiing)
{
 //cout << "copy constructor Changrp: ";
 //cout << copiing.mID << " " << copiing.mTSrange << " " << copiing.mContrID << endl; 
 //IMPORTANT - deep copy of Serial Iface pointer
 //oparate with null must ne very careful and perfect 
 if (copiing.mpSerIface != NULL) {
  //cout << "mpSerIface deep copy" << endl;
  mpSerIface=new GIface(*copiing.mpSerIface);
 } else {
  mpSerIface=NULL;
 }
 mID = copiing.mID;
 mTSrange = copiing.mTSrange;
 mContrID= copiing.mContrID;
} 

//---------------------------------------------------------------------------
/**
 * Destructor
 */
Changrp::~Changrp()
{
 //delete local copy of serial interface object 
 if (mpSerIface!=NULL) delete mpSerIface;
}

//---------------------------------------------------------------------------
/**
 * Check if corresponding serial interface is vector of serial, if yes
 * assign it to mpSerIflink.
 * @return true, if interface link was found, false if not 
 */
bool Changrp::makeSerIfacelink(std::vector<GIface>& rSerIfacesV)
{
 for(vector<GIface>::iterator iter=rSerIfacesV.begin();iter<rSerIfacesV.end();iter++)
 {
  if (isSerID(iter->getName())) {
   //IMPORTANT we must make deep copy, vector referenced in parameter can be only
   //temporary, be aware on memory leaks
   if (mpSerIface != NULL) {
    throw logic_error("Changrp::makeSerIfacelink - can't set serial iface for chnnel-group more then once!");
    return false;    
   } 
   //deep copy
   //cout << "Changrp::makeSerIfacelink - copiing " << iter->getName() << endl; 
   mpSerIface=new GIface(*iter);
   return true;
  }
 }
 return false;
}

//---------------------------------------------------------------------------
/**
 * check if serial interface id (e.g 3/2/1:0) is corresponded with channel group 
 * and controller id 
 */
bool Changrp::isSerID(std::string serID)
{
 //local serial ID made from mContrID
 string mkserID=makeSerID();
 //cout << "Changrp::isSerID mkserID: " << mkserID << "serID: " << serID << endl;
 if (mkserID==serID) return true;
 return false;
}

//---------------------------------------------------------------------------
/**
 * make serial interface ID from controller - mContrID and channel group ID - mID
 */
std::string Changrp::makeSerID()
{
 return mContrID+":"+Utils::inttostr(mID);
}

//---------------------------------------------------------------------------
/**
 * The count is derived from the mTSrange string e.g. for 1,3-16 => 17 used
 * timeslots
 */
unsigned int Changrp::getUsedTSCount() const
{
 //unframed - no timeslots are free
 if (Utils::regFirstMatch("unframed",mTSrange)) {
  return 0;
 }
 
 //get all timeslot ranges, these are separated by comma, examle
 //for mTSrange="1,10-23" return a vector with the elements [0] = "1", [1] = "10-32"
 vector<string> tsRanges;
 Utils::Tokenize(mTSrange,tsRanges,",");

 unsigned int usedTS=0;
 //for each timeslot range, get number of used timeslots
 for (vector<string>::iterator iter=tsRanges.begin();iter<tsRanges.end();iter++) {
  //cout << "tsrange: " << *iter  << endl;
  usedTS+=getUsedTSFromTSRange(*iter);
 }

 return usedTS;
}

//---------------------------------------------------------------------------
/**
 * One timeslot range is : "1" or "3-9", not "1,3-9", this is two-timeslot
 * range, which is tokenized in the getUsedTSCount.
 */
unsigned int Changrp::getUsedTSFromTSRange(std::string& tsRange)
{
 //no "-" presented, only one timeslot
 if (tsRange.find("-") == string::npos) {
  return 1;
 }

 string::size_type delimPos=tsRange.find("-");
 string beginTSstr=tsRange.substr(0,delimPos);
 string endTSstr=tsRange.substr(delimPos+1,tsRange.size()-delimPos);

 //cout << "begin: " << beginTSstr << " end: " << endTSstr << endl;
 return ( Utils::strtoint(endTSstr) - Utils::strtoint(beginTSstr) +1 );
}
