/**
 * @file
 * @brief implementation of the CiscoBox class
 * @author Miroslav Kubelik
 * @date 2007
 */

//STDC++ and STL Headers
#include <string>
#include <vector>
#include <iostream>
#include <stdexcept>
#include <sstream>

//C and local headers
#include <time.h>
#include "CiscoBox.h"
#include "Utils.h"

using namespace std;

//---------------------------------------------------------------------------
/**
 * Destructor, free (clear) all vectors for controllers etc.
 */
CiscoBox::~CiscoBox()
{
 mE1contrV.clear();
 mE3contrV.clear();
 mAllIfaceV.clear();
}

//---------------------------------------------------------------------------------------
/**
 * Copy constructor
 * !!Be aware!! that all elements in the vectors have correctly defined copy constructors
 */
CiscoBox::CiscoBox(const CiscoBox& copiing)
{
 //it is necessary, that all vector elements types have corretly defined
 //copy constructor
 mE1contrV = copiing.mE1contrV;
 mE3contrV = copiing.mE3contrV;
 mSonetcontrV = copiing.mSonetcontrV;
 mAllIfaceV = copiing.mAllIfaceV;

 mID = copiing.mID;
 mIPaddr = copiing.mIPaddr;
 mHostname= copiing.mHostname;
}

//---------------------------------------------------------------------------
/**
 * check for type of controller and add it to corresponding array
 * COPY of object is saved to array, because push_back calls copy constructor
 * used dynamic type recast     
 */ 
void CiscoBox::addController(Controller* pContr)
{
 //cout << "CiscoBox::addController" << endl;
 if (pContr==NULL) return;

 //E1 Controller
 E1Controller* pE1Contr=dynamic_cast<E1Controller*>(pContr);
 if (pE1Contr!=NULL) {
  mE1contrV.push_back(*pE1Contr);
  return;
 }
 //E3 Controller
 E3Controller* pE3Contr=dynamic_cast<E3Controller*>(pContr);
 if (pE3Contr!=NULL) {
  mE3contrV.push_back(*pE3Contr);
  return;
 }
 //Sonet Controller
 SonetController* pSonetContr=dynamic_cast<SonetController*>(pContr);
 if (pSonetContr!=NULL) {
  mSonetcontrV.push_back(*pSonetContr);
  return;
 }
 //unknow controller type, make exception
 ostringstream osstrm;
 osstrm << "CiscoBox::addController - unknow controller type - id: " << pContr->getName() << endl;
 throw logic_error(osstrm.str());
}

//---------------------------------------------------------------------------
/**
 * add object (not pointer) of GIface to our all interfaces array
 * COPY of object is saved to array, because push_back calls copy constructor
 * it couses, that GSubIf objects are automatically converted to parent GIface
 * object and saved to vector 
 */ 
void CiscoBox::addIface(GIface* pIface)
{
 if (pIface==NULL) return;
 switch( pIface->getType() )
 {
  case eUndef:
   cout << "Error: CiscoBox::addIface: undefined controller type for iface " << pIface->getName() << endl;
   return; 
 	 break;
 	default:
 	 //all interfaces into one array
 	 //copy constructor is called!, copy of object is saved to vector 
   mAllIfaceV.push_back(*pIface);
   break;
 }
}

//---------------------------------------------------------------------------
/**
 * get interface object defined by interface name and string type 
 * type as string in "encoded" to mapped controller type (typedef ::ContrType) and
 * CiscoBox::getIface(string ifName,ContrType type) is called 
 */  
GIface* CiscoBox::getIface(std::string ifName,std::string iftype)
{
 //get controller type mapped to iftype string
 ContrType type=GIface::getMapType(iftype);
 return getIface(ifName,type);
}

//---------------------------------------------------------------------------
/**
 * get interface object defined by interface name and type typedef::ContrType
 */  
GIface* CiscoBox::getIface(std::string ifName,ContrType type)
{
 if (type == eUndef) {
  cout << "Error: CiscoBox::getIface - undefined type for requested interface " << ifName << endl;
  return NULL;
 } 
 
 for (vector<GIface>::iterator iter=mAllIfaceV.begin();iter<mAllIfaceV.end();iter++)
 {
  //cout << "CiscoBox::getIface: check for " << ifName << " " << iter->getName() << endl;
  if (iter->getName()==ifName) {
   //cout << "CiscoBox::getIface: Ifname for " << ifName << " found" << endl;
   return &(*iter);
  }
 }
 return NULL;
}


//------------------------------------------------------------------------------
/**
 * get all interfaces with same type from all interfaces vector
 * type is select by controller type typedef ::ContrType. Controller is base class for all
 * interfaces. Every interface must set Controller::mType as typedef ::ContrType.     
 */
std::vector<GIface> CiscoBox::getAllIfOneType(ContrType type) const
{
 vector<GIface> AllIfV;
 for (vector<GIface>::const_iterator iter=mAllIfaceV.begin();iter<mAllIfaceV.end();iter++)
 {
  if (iter->getType()==type) {
   //add copy of object 
   AllIfV.push_back(*iter);
  }
 }
 return AllIfV;
}

//---------------------------------------------------------------------------
/**
 * Make connection between all channelgroups and corresponding GIface interface object.
 * Each GIface object is saved as copy to a Changrp object.
 */
void CiscoBox::makeChangrpSerlinks()
{
 try 
 { 
  //vector of Serial Ifaces (without Subifs)
  vector<GIface> SerIfV=getAllIfOneType(eSerialIf);
  
  //for all E1 controllers
  for (vector<E1Controller>::iterator iter=mE1contrV.begin();iter<mE1contrV.end();iter++)
  {
   iter->makeChangrpSerlinks(SerIfV);
  }
  //for all E3 controllers
  for (vector<E3Controller>::iterator iter=mE3contrV.begin();iter<mE3contrV.end();iter++)
  {
   iter->makeChangrpSerlinks(SerIfV);
  }
  //for all Sonet controllers
  for (vector<SonetController>::iterator iter=mSonetcontrV.begin();iter<mSonetcontrV.end();iter++)
  {
   iter->makeChangrpSerlinks(SerIfV);
  }
 } catch (const exception& e) {
   cout << "Got an exception: " << e.what() << endl;
   exit (EXIT_FAILURE);
 }
}

//---------------------------------------------------------------------------
/**
 * Print all controllers and interfaces, mostly for debug.
 */
void CiscoBox::printDebuginfo() const
{
 cout << "CiscoBox debug info " << endl;
 cout << "name " << getHostname() << endl;
 cout << "E1controllers: " << mE1contrV.size() << endl;
 /*for (vector<E1Controller>::iterator iter=mE1contrV.begin();iter<mE1contrV.end();iter++)
 {
  iter->printDebuginfo();
 }
 */
 cout << "E3Controllers: " << mE3contrV.size() << endl;
 /*for (vector<E3Controller>::iterator iter=mE3contrV.begin();iter<mE3contrV.end();iter++)
 {
  iter->printDebuginfo();
 }
 */
 cout << "SonetControllers: " << mSonetcontrV.size() << endl;
 /*for (vector<SonetController>::iterator iter=mSonetcontrV.begin();iter<mSonetcontrV.end();iter++)
 {
  iter->printDebuginfo();
 }
 */
 cout << "AllInterfaces: " << mAllIfaceV.size() << endl;
 /*for (vector<GIface>::iterator iter=mAllIfaceV.begin();iter<mAllIfaceV.end();iter++)
 {
  iter->printDebuginfo();
 }
 */
 vector<GIface> IfV=getAllIfOneType(eSerialIf);
 cout << "SerialInterfaces: " << IfV.size() << endl;
 /*
 for (vector<GIface>::iterator iter=IfV.begin();iter<IfV.end();iter++)
 {
  iter->printDebuginfo();
 }
 */
}
