/**
 * @file
 * @brief declaration of the CiscoBox class
 * @author Miroslav Kubelik
 * @date 2007
 */

#ifndef CISCOBOX_H
#define CISCOBOX_H

/* necessary declaration for class definition */
#include <vector>
#include <string>
#include "E3Controller.h"
#include "E1Controller.h"
#include "GIface.h"
#include "SonetController.h"

/**
 * @brief class for the one Cisco device
 */
class CiscoBox
{
 private:
  /**
   * @brief device ID e.g router1
   */
  std::string mID;

  /**
   * @brief IP address
   * IP address or DNS name
   */
  std::string mIPaddr;

  /**
   * @brief hostname
   * often same as mID, is automatically set in parsing as "hostaname" parametr in
   * device configuration or can be set via setHostname
   */     
  std::string mHostname;

  /**
   * @brief all E1 controllers
   * vector for E1 Controllers
   */
  std::vector<E1Controller> mE1contrV;

  /**
   * @brief all interfaces
   * all interfaces (serials, serialsubifs, atm, atmsubifs ...) are only in this one array
   * selected by ContrType, because base class in Controller
   */      
  std::vector<GIface> mAllIfaceV;

  /**
   * @brief all E3 controllers
   * vector for E3 Controllers
   */
  std::vector<E3Controller> mE3contrV;

  /**
   * @brief all Sonet controllers
   * vector for Sonet Controllers
   */
  std::vector<SonetController> mSonetcontrV;
 
 public:
  CiscoBox(){}

  /** @brief copy constructor, all elemets types in the vectors must have
   * correctly defined copy constructors
   */
  CiscoBox(const CiscoBox& copiing);
  
  ~CiscoBox();
  void setHostname(std::string name) {mHostname=name;}
  void setID(std::string str) {mID=str;}
  /**
   * const method, doesn't modify "inner state" of object
   */    
  std::string getID() const {return mID;}
  std::string getHostname() const {return mHostname;}
  GIface* getIface(std::string ifName,std::string iftype);
  GIface* getIface(std::string ifName,ContrType type);
  /**
   * inspect method (const), return reference to const vector
   */     
  const std::vector<E1Controller>& getAllE1() const {return mE1contrV; }
  const std::vector<E3Controller>& getAllE3() const {return mE3contrV; }
  const std::vector<SonetController>& getAllSonet() const {return mSonetcontrV; }

  /** @brief add controller to some controllers vectors */
  void addController(Controller* pContr);  
  /** @brief add interface to all interfaces vector */
  void addIface(GIface* pIface);
  /** @brief get interfaces of one type */
  std::vector<GIface> getAllIfOneType(ContrType type) const;
  /** @brief build "connection" between serial interfaces and channel-groups */
  void makeChangrpSerlinks();
  void printDebuginfo() const;
};

#endif
