/**
 * @file
 * @brief implementation of the CiscoconfParser class
 * @author Miroslav Kubelik
 * @date 2007
 */
 
//STDC++ and STL Headers
#include <string>
#include <vector>
#include <iostream>
#include <sstream>

//C and local headers
#include "regexp/regexp/Matcher.h"
#include "regexp/regexp/Pattern.h"
#include "E1Controller.h"
#include "E3Controller.h"
#include "SonetController.h"
#include "GIface.h"
#include "GSubif.h"
#include "CiscoBox.h"
#include "CiscoconfParser.h"
#include "Utils.h"

const unsigned int CiscoconfParser::PARS_STATUS_UNDEF = 1;
const unsigned int CiscoconfParser::PARS_STATUS_CONTR = 2;
const unsigned int CiscoconfParser::PARS_STATUS_IFACE = 3;

using namespace std;

//---------------------------------------------------------------------------
/**
 * constructor
 */
CiscoconfParser::CiscoconfParser()
{
 mpCiscoBox=NULL;
}

//---------------------------------------------------------------------------
/**
 * destructor
 */
CiscoconfParser::~CiscoconfParser()
{
 delete mpCiscoBox;
}

//---------------------------------------------------------------------------
/**
 * parse cisco config as text line by line
 * @param rInS reference to input string stram with cisco configuration file
 */
void CiscoconfParser::parse(std::istream& rInS)
{
 Controller* pContr=NULL;
 GIface* pIface=NULL;
 //initial parser status is undef
 mStatus=PARS_STATUS_UNDEF;
 string line;

 mpCiscoBox=new CiscoBox();

 while(getline(rInS,line).good()) {
  //cout << "line " << line << "status " << mStatus << endl;
  //controller section begin, concrete cntroller is returned as pointer to
  //base controller class (thanks polymorphism)
  if (this->isBegContr(line) && mStatus==PARS_STATUS_UNDEF) {
   mStatus=PARS_STATUS_CONTR;
   pContr=factoryContrObj(line);
   if (pContr == NULL) {
    mStatus=PARS_STATUS_UNDEF;
   } else {
    pContr->parsContrSection(line);
   } 
   continue;
  }
  if (mStatus==PARS_STATUS_CONTR) {
   pContr->parsContrSection(line);
  }
  //interface section begin
  if (Utils::regFirstMatch("^interface \\S+",line) && mStatus==PARS_STATUS_UNDEF) {
   //create interface (this is !Pointer! can be GSubif or GIface - inheritence)
   pIface=this->factoryIfObj(line);
   if (pIface == NULL) {
    mStatus=PARS_STATUS_UNDEF;
   } else {
    mStatus=PARS_STATUS_IFACE; 
    pIface->parsSerSection(line);
   } 
   continue;
  }
  if (mStatus==PARS_STATUS_IFACE) {
   pIface->parsSerSection(line);
   //not continue, end of section must be matched
  }
  if (this->isEndSection(line)) {
   switch(mStatus) {
    //end of controller section, add into device object
    case (PARS_STATUS_CONTR):
     if (pContr!=NULL) {
      mpCiscoBox->addController(pContr);
      delete pContr;pContr=NULL;
     } 
     break;
    case (PARS_STATUS_IFACE):
     if (pIface!=NULL) {
      //if interface is Subinterface, add him to main interface
      GSubif* pSubif=dynamic_cast<GSubif*>(pIface);
      if (pSubif!=NULL) {
       //get main interface ID
       string mainIfID=pSubif->getMainIfID();
       //get main iface object, type is same as subif 
       GIface* pMainIface=mpCiscoBox->getIface(mainIfID,pIface->getType());
       if (pMainIface==NULL) {
        cout << "Error: Main interface for " << pSubif->getName() <<  " doesn't exist" << endl;
       } else {
        pMainIface->addCopySubifp(pSubif);
       }
      }
      mpCiscoBox->addIface(pIface);
      delete pIface;pIface=NULL;
     } 
     break;
    default:
     break;
   }
   mStatus=PARS_STATUS_UNDEF;
   continue;
  }
  if (mStatus==PARS_STATUS_UNDEF) {
   parsedevinfo(line);
  }
 }
 
 delete pContr;
 delete pIface;
}

//---------------------------------------------------------------------------
/**
 * @brief check begin of controller section
 */
bool CiscoconfParser::isBegContr(std::string& rline)
{
 if (Utils::regFirstMatch("^controller \\S+",rline)) {
  return true;
 }
 return false;
}

//---------------------------------------------------------------------------
/**
 * @brief check end of the section
 *
 * A section in the cisco configuration is ended with '!'.
 */
bool CiscoconfParser::isEndSection(std::string& rline)
{
 if (Utils::regFirstMatch("^!",rline)) {
  return true;
 }
 return false;
}
//---------------------------------------------------------------------------
/**
 * @brief Factory method for interfaces.
 *
 * Return interface object on basis "^interface xxx" line.
 * If it's line with subifs, GSubif object pointer is returned
 * (see GIface::CreateIface factory)
 * 
 * workaround: ATM IMA interface are ignored  
 */
GIface* CiscoconfParser::factoryIfObj(std::string& line)
{
 GIface *pIface;
 ostringstream osstrpattern;
 
 //serial iface, including subifs
 osstrpattern << "^interface " << GIFACE_SERIALSTR << "\\S+";
 if (Utils::regFirstMatch(osstrpattern.str(),line)) {
  pIface=GIface::CreateIface(line,mpCiscoBox,GIFACE_SERIALSTR);
  return pIface;
 }
 
 //clear stringstream
 osstrpattern.str("");
 
 //workaround ignore ATM IMA Ifaces
 osstrpattern << "^interface " << GIFACE_ATMSTR << ".*ima";
 if (Utils::regFirstMatch(osstrpattern.str(),line)) {
  return NULL;
 }
 
 //clear stringstream
 osstrpattern.str("");
 
 //ATM interface including subifs
 osstrpattern << "^interface " << GIFACE_ATMSTR << "\\S+";
 if (Utils::regFirstMatch(osstrpattern.str(),line)) {
  pIface=GIface::CreateIface(line,mpCiscoBox,GIFACE_ATMSTR);
  //cout << "ATM Iface created: line " << line << endl; 
  return pIface;
 }

 //clear stringstream
 osstrpattern.str("");
 
 //POS interface including subifs
 osstrpattern << "^interface " << GIFACE_POSSTR << "\\S+";
 if (Utils::regFirstMatch(osstrpattern.str(),line)) {
  pIface=GIface::CreateIface(line,mpCiscoBox,GIFACE_POSSTR);
  //cout << "POS Iface created: line " << line << endl; 
  return pIface;
 }
 return NULL;
}

//---------------------------------------------------------------------------
/**
 * @brief Factory for controllers.
 *
 * Return controller object on basis "^controller xxxx" line
 * checks type of controller and return corresponding Controller child object pointer 
 * (E1Controller, E3Controller etc.) 
 */
Controller* CiscoconfParser::factoryContrObj(std::string& line)
{
 Controller *pContr;
 ostringstream osstrpattern;
 
 //E1 Controller
 osstrpattern << "^controller E1 ";
 if (Utils::regFirstMatch(osstrpattern.str(),line)) {
  pContr=new E1Controller();
  return pContr;
 }
 
 //clear stringstream
 osstrpattern.str("");
 
 //E3Controller
 osstrpattern << "^controller E3 ";
 if (Utils::regFirstMatch(osstrpattern.str(),line)) {
  pContr=new E3Controller();
  return pContr;
 }

 //clear stringstream
 osstrpattern.str("");
 
 //Soner Controller
 osstrpattern << "^controller SONET ";
 if (Utils::regFirstMatch(osstrpattern.str(),line)) {
  pContr=new SonetController();
  return pContr;
 }
 
 return NULL;
}

//---------------------------------------------------------------------------
/**
 * @brief get main device informations
 */
void CiscoconfParser::parsedevinfo(std::string& rline)
{
 Pattern* pPattern=NULL;
 Matcher* pMatcher=NULL;

 //cout << "CiscoconfParser::ParsE1contr: line : " << rline << endl;

 //get device hostname
 if (mpCiscoBox->getHostname().empty()) {
  pPattern=Pattern::compile("hostname (\\S+)");
  pMatcher = pPattern->createMatcher(rline);
  if (pMatcher->findFirstMatch()) {
   mpCiscoBox->setHostname(pMatcher->getGroup(1));
  }
  delete pPattern;delete pMatcher;
 }
}
