/*
* implementation of E1Controller class
*/

//STDC++ and STL Headers
#include <string>
#include <vector>
#include <iostream>
#include <sstream>
#include <stdexcept>

//C and local headers
#include "regexp/regexp/Matcher.h"
#include "regexp/regexp/Pattern.h"
#include "E1Controller.h"
#include "Utils.h"

using namespace std;

//---------------------------------------------------------------------------
/**
 * Constructor set mType as E1controller
 */
E1Controller::E1Controller()
{
 if (!mChangrpV.empty()) {
  mChangrpV.clear();
 }
 mType=eE1contr;
}

//---------------------------------------------------------------------------
/**
 * Destructor, free all channel groups
 */
E1Controller::~E1Controller()
{
 //bacause vector items are objects, not pointers, destructors are
 //called automaticly in clear method
 mChangrpV.clear();
}

//---------------------------------------------------------------------------
/**
 * get Channel groups count in controller
 */
int E1Controller::ChangrCnt()
{
 return (int)mChangrpV.size();
}

//---------------------------------------------------------------------------
bool E1Controller::isUnframed() const
{
 if (mUnframed) return true;
 else return false;
}

//---------------------------------------------------------------------------
/**
 * set E1 controller framing (crc-4, no-crc...)
 * @param str controller framing as text  
 */
void E1Controller::setFraming(std::string str)
{
 mframing=str;
}

//---------------------------------------------------------------------------
/**
 * set E1 clock source
 */
void E1Controller::setClockSrc(std::string str)
{
 mclockSrc=str;
}

//---------------------------------------------------------------------------
/**
 * add channel group to controller
 * @param id channel group ID
 * @param range timeslot range (1-13 3,1-13,16 etc)
 */
Changrp& E1Controller::addChangr(int id,std::string range)
{
 //cout << "E1Controller::addChangr - " << getName() << " " << id << " " << range << endl; 
 Changrp* pChangrp=new Changrp(id,range);
 mChangrpV.push_back(*pChangrp);
 delete pChangrp;
 //cout << "return E1Controller::addChangr " << endl;
 return mChangrpV.back(); 
}

//---------------------------------------------------------------------------
/**
 * print info about controller - for debug
 */
void E1Controller::printDebuginfo() const
{
 cout << "E1 controller: " << mName << endl;
 cout << "descr: " << mDescr << endl;
 cout << "channelgroups: " << mChangrpV.size() << endl;
 cout << "CID " << getCID(getDescr()) << endl;
 cout << "unframed " << ( isUnframed() ? "true " : "false" )  << endl;
}

//---------------------------------------------------------------------------
/**
 * get main E1 properties from E1 controller section lines
 * called periodically from  
 */
void E1Controller::parsContrSection(std::string& rline)
{
 Pattern* pPattern;
 Matcher* pMatcher;

 bool ParserStop=false;

 //get controller ID
 pPattern=Pattern::compile("^controller E1 (\\S+)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  //cout << "controller ID line" << rline << " mathed " << endl;
  string matchedstr=pMatcher->getGroup(1);
  //cout << "matched string " << matchedstr << endl;
  setName(matchedstr);
  ParserStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParserStop) return;

 //get description
 pPattern=Pattern::compile("description (.*)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  //cout << "E1 controller description " << rline << " mathed " << endl;
  string matchedstr=pMatcher->getGroup(1);
  //cout << "E1 matched string " << matchedstr << endl;
  setDescr(matchedstr);
  ParserStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParserStop) return;
 
//get framing
 pPattern=Pattern::compile("framing (.*)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  string matchedstr=pMatcher->getGroup(1);
  //cout << "E1 framing " << matchedstr << endl;
  setFraming(matchedstr);
  ParserStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParserStop) return;
 
//get clock source
 pPattern=Pattern::compile("clock source (.*)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  string matchedstr=pMatcher->getGroup(1);
  //cout << "E1 clock source " << matchedstr << endl;
  setClockSrc(matchedstr);
  ParserStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParserStop) return;

 //unframed channel groups section
 pPattern=Pattern::compile("channel-group (\\d+) unframed$");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  //cout << "unframed E1 channel-group " << rline << endl;
  Changrp& rChangrp=addChangr(Utils::strtoint(pMatcher->getGroup(1)),"unframed");
  rChangrp.setContrID(getContrID());
  setUnframed(true);
  ParserStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParserStop) return;

 //unframed section in Sub E1 controller (e.g. in Sonet controller: tug-2 1 e1 1 unframed )
 pPattern=Pattern::compile(" unframed$");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  //cout << "unframed E1 channel-group " << rline << endl;
  Changrp& rChangrp=addChangr(0,"unframed");
  rChangrp.setContrID(getContrID());
  setUnframed(true);
  ParserStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParserStop) return;
 
 //framed channel groups section
 //nedded [ ]+ before the "timeslosts" word, because on some devices (Catalyst 8500), there
 //are more then one space
 pPattern=Pattern::compile("channel-group (\\d+)[ ]+timeslots (\\S+)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  //cout << "framed E1 channel-group " << rline << endl;
  Changrp& rChangrp=addChangr(Utils::strtoint(pMatcher->getGroup(1)),pMatcher->getGroup(2));
  rChangrp.setContrID(getContrID());
  setUnframed(false);
  ParserStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParserStop) return; 
}

//---------------------------------------------------------------------------
/**
 * get controller ID - return "numeric" part of controller name e.g. 11/0/0.1/3/6/2
 * it's needed because sub E1 in Sonet controllers has name as ":TU59 11/0/0.1/3/6/2"
 */
std::string E1Controller::getContrID()
{
 string contrID="";
 Pattern* pPattern=Pattern::compile("\\d+[/\\d.]*$");
 Matcher* pMatcher = pPattern->createMatcher(getName());
 if (pMatcher->findFirstMatch()) {
  contrID=pMatcher->getGroup();
 } else {
  cout << "Error contrID not found for " << getName() << endl;
 }
 delete pPattern;delete pMatcher;
 return contrID;
}
      
//---------------------------------------------------------------------------
/**
 * make links between channel groups in controller and corresponding serial interface 
 */
void E1Controller::makeChangrpSerlinks(std::vector<GIface>& rSerIfacesV)
{
 for (vector<Changrp>::iterator iter=mChangrpV.begin();iter<mChangrpV.end();iter++)
 {
  Changrp& rChangrp=(*iter);
  if (!rChangrp.makeSerIfacelink(rSerIfacesV)) {
   ostringstream osstrm;
   osstrm << "Error: Controller:" << mName << " channel group: " << rChangrp.getID() ;
   osstrm << " corresponded serial interface " << rChangrp.makeSerID() << " not found" << endl;
   //cout << osstrm.str();  
   throw logic_error(osstrm.str());
  }
 }
}
