/**
 * @file
 * @brief implementation of the E3Controller class
 * @author Miroslav Kubelik
 * @date April 2007
 */

//STDC++ and STL Headers
#include <iostream>
#include <string>
#include <stdexcept>

//C and local headers
#include "regexp/regexp/Matcher.h"
#include "regexp/regexp/Pattern.h"
#include "E3Controller.h"
#include "Utils.h"
#include "E1Controller.h"

using namespace std;

//---------------------------------------------------------------------------
/**
 * normal constructor
 */
E3Controller::E3Controller()
{
  mpE1ControllerA=new E1Controller[mcNumSubE1];
}

//---------------------------------------------------------------------------
/**
 * copy constructor, needed, because we do push_back() this class with pointer to vector
 */
E3Controller::E3Controller(const E3Controller& m) : Controller(m)
{
 //create dynamic array
  mpE1ControllerA=new E1Controller[mcNumSubE1];
  for (int i=0;i<mcNumSubE1;i++) {
   mpE1ControllerA[i] = m.mpE1ControllerA[i];
  }
}

//---------------------------------------------------------------------------
/**
 * Destructor
 */
E3Controller::~E3Controller()
{
 delete [] mpE1ControllerA;
}

//---------------------------------------------------------------------------
/**
 * get properties from E3 controller line
 */
void E3Controller::parsContrSection(std::string& rline)
{
 Pattern* pPattern;
 Matcher* pMatcher;

 bool ParseStop=false;
 
 //get controller ID
 pPattern=Pattern::compile("^controller E3 (\\S+)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  //cout << "controller ID line" << rline << " mathed " << endl;
  string matchedstr=pMatcher->getGroup(1);
  //cout << "matched string " << matchedstr << endl;
  setName(matchedstr);
  ParseStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParseStop) return;

 //get whole E3 controller description
 pPattern=Pattern::compile(" description (.*)$");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  string matchedstr=pMatcher->getGroup(1);
  setDescr(matchedstr);
  ParseStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParseStop) return;
 
 //subE1 controller
 pPattern=Pattern::compile("\\s+e1 (\\d+)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  //cout << "subE1 in e3 controller matched" << rline << endl;
  string subE1idstr=pMatcher->getGroup(1);
  //cout << "subE1 " << subE1idstr << endl;
  int subE1id=Utils::strtoint(subE1idstr);
  if (subE1id>0 && subE1id<(mcNumSubE1+1)) {
   //sub E1 array index is between 0-15
   E1Controller& rSubE1contr=mpE1ControllerA[--subE1id];
    //IMPORTANT: set controller name first, because from controller name is
   //derived other info (channel-group controller ID) in parsContrSection  
   if (rSubE1contr.getName().empty()) {
    rSubE1contr.setName( getSubE1Name(subE1id+1) );
    //rSubE1contr.setName(mName+"/"+subE1idstr);
   }
   rSubE1contr.parsContrSection(rline);
  }
  ParseStop=true;
 }
 delete pPattern;delete pMatcher;
}

//---------------------------------------------------------------------------
void E3Controller::printDebuginfo()
{
 cout << "E3controller " << mName << endl;
 for (int i=0;i<mcNumSubE1;i++) {
  E1Controller& rSubE1contr=mpE1ControllerA[i];
  rSubE1contr.printDebuginfo();
 }
}

//---------------------------------------------------------------------------
/**
 * make links between channel groups in controller and corresponding serial interface 
 */
void E3Controller::makeChangrpSerlinks(std::vector<GIface>& rSerIfacesV)
{
 for (int i=0;i<mcNumSubE1;i++) {
  E1Controller& rSubE1contr=mpE1ControllerA[i];
  rSubE1contr.makeChangrpSerlinks(rSerIfacesV);
 }
}

//---------------------------------------------------------------------------
/**
 * get subE1 name for the concrete subE1 count number
 *
 * E.g. A whole E3 constroller has name 1/0, for second subE1, subid = 2, this
 * function returns 1/0/2
 * @throws out_of_range if subID is not in E3 subid range
 */
std::string E3Controller::getSubE1Name(unsigned int subE1id) const
{
 if (subE1id>0 && subE1id<(mcNumSubE1+1)) {
  return (mName+"/"+Utils::inttostr(subE1id));
 }
 throw out_of_range("subE1 in E3 must be in interval <1-16>");
}
