/**
 * @file
 * @brief implementation for the generic inteface class
 * @author Miroslav Kubelik
 * @date 2007
 */

//STDC++ and STL Headers
#include <iostream>
#include <string>
#include <string>
#include <stdexcept>

//C and local headers
#include "regexp/regexp/Matcher.h"
#include "regexp/regexp/Pattern.h"
#include "GIface.h"
#include "GSubif.h"
#include "CiscoBox.h"

using namespace std;

//---------------------------------------------------------------------------
/**
 * constructor
 */
GIface::GIface() : mHaveSubIf(false),mConfIfDeter(""),mIPaddr("") 
{
 mSubIfVp.clear();
}

//---------------------------------------------------------------------------
GIface::GIface(std::string iftype) : mHaveSubIf(false),mIPaddr("")
{
 setConfIfDeter(iftype);
 mSubIfVp.clear();
}

//---------------------------------------------------------------------------
void GIface::setConfIfDeter(std::string str)
{
 mConfIfDeter = str;
 setType();
}

//---------------------------------------------------------------------------
void GIface::setType()
{
 ContrType type=GIface::getMapType(mConfIfDeter);
 if (type == eUndef) {
  throw logic_error("GIface::setType - can't set Iface type, unknow determinant "+mConfIfDeter);
  return;
 } else {
  mType=type;
 }
}

//---------------------------------------------------------------------------
/**
 * get ContrType (defined in base Controller class), which is corresponding to
 * interfece type string in configuration (e.g Serial, ATM ...)
 */   
ContrType GIface::getMapType(std::string str)
{
 if (str == GIFACE_SERIALSTR) {
  return eSerialIf;
 }
 if (str == GIFACE_ATMSTR) {
  return eATMIf;
 }
 if (str == GIFACE_POSSTR ) {
  return ePOSif;
 }
 return eUndef;
}

//---------------------------------------------------------------------------
/**
 * copy constructor, nedded if use "dynamic memory" structures
 * in class members (e.g. vector of pointers, arrays of pointers)
 */
GIface::GIface(const GIface& m) : Controller(m)
{
 //cout << "copyconstuctor for GIface " << m.mName << endl; 
 //create copy of vector, just copy pointers.
 mSubIfVp.clear();
 //ierator must be const_iterator, because GIface& m is const
 if (m.mSubIfVp.size()>0) {
  //cout << "copiing mSubIfVp - " << m.mSubIfVp.size() << endl;    
  for(vector<GSubif*>::const_iterator iter=m.mSubIfVp.begin();iter<m.mSubIfVp.end();iter++)
  {
   GSubif* pSubif=(*iter);
   if (pSubif!=NULL) {
    //cout << "deep copy of " << pSubif->getName() << endl;
    addCopySubifp(pSubif);
   } else {
    //cout << "pSubif is null " << endl;
   } 
  }
 }
 //copy members data
 mIPaddr=m.mIPaddr;
 mHaveSubIf=m.mHaveSubIf;
 mConfIfDeter=m.mConfIfDeter;
 //cout << "copy const for SerialIface finished" << endl;
}

//---------------------------------------------------------------------------
GIface::~GIface()
{
 //cout << "destructor for SerialIface " << getName() << endl;
 for(vector<GSubif*>::iterator iter=mSubIfVp.begin();iter<mSubIfVp.end();iter++)
 {
  //cout << "deleting Subif"  << (*iter)->getName() << endl;
  delete (*iter);
 }
 mSubIfVp.clear();
}

//---------------------------------------------------------------------------
void GIface::parsSerSection(std::string& rline)
{
 Pattern* pPattern=NULL;
 Matcher* pMatcher=NULL;

 bool ParseStop=false;
 
 //get description
 pPattern = Pattern::compile("description (.*)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  string matchedstr=pMatcher->getGroup(1);
  //cout << "interface description " << matchedstr << endl;
  setDescr(matchedstr);
  ParseStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParseStop) return;

 //get Ip address
 pPattern = Pattern::compile("ip address ([\\d.]+ [\\d.]+)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  string ipAddr=pMatcher->getGroup(1);
  //cout << "interface IP address " << ipAddr << endl;
  setIPaddr(ipAddr);
  ParseStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParseStop) return;

 //get name of interface, subif included
 string pattrn="interface "+mConfIfDeter+"([\\d.:/]+)";
 pPattern = Pattern::compile(pattrn);
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  string matchedname=pMatcher->getGroup(1);
  //cout << "line: " << rline << endl;
  //cout << "interface name " << matchedname << endl;
  setName(matchedname);
  ParseStop=true;
 }
 delete pPattern;delete pMatcher;

}

//---------------------------------------------------------------------------
/*
 * factory method, if interface is subif, create subif and return him, if not
 * return only serial Iface 
 * line is interface line (interface Serial1/0/2:1), can be line for
 * serial subinterface  
 */
GIface* GIface::CreateIface(std::string& rline,CiscoBox* pCiscoBox,std::string iftype)
{
 //try to get MainifID from subinterface, if not empty, inteface is
 //subinterface
 string mainIfID=GSubif::getMainIfIDfromIfaceline(iftype,rline);
 if (!mainIfID.empty()) {
  //cout << "GIface::CreateIface: subif detect in " << rline << endl;
  GSubif* pSubif=new GSubif();
  //check if main interface already exits
  GIface* pMainIface=pCiscoBox->getIface(mainIfID,iftype);
  if (pMainIface==NULL) {
   cout << " Error: GIface::CreateIface - main interface " << mainIfID << " for "  << rline <<  " doesn't exist" << endl;
   delete pSubif;
   return NULL;
  } 
  //set "have subif" for main interface
  pMainIface->setHaveSubIf(true);
  //set string determinant for subif
  pSubif->setConfIfDeter(iftype); 
  return pSubif;
 }

 //check for interface without SubIf
 string regpattern="interface "+iftype+"[\\d./:]+";
 Pattern* pPattern=Pattern::compile(regpattern);
 Matcher* pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  //cout << "Normal seriface in line " << rline << " matched" << endl;
 }
 delete pPattern;delete pMatcher;
 //set string determinant is important
 return new GIface(iftype);
}

//---------------------------------------------------------------------------
/**
 * trim all over interface ID (e.g Serial1/0/1:0.12 point-to-point -> Serial1/0/1:0.12)
 * use trimed line with end direct aftre interface ID is very handy in 
 * parsing main intreface from subif and subif id see GSubif::parsSerSection()
 * and GSubif::getMainIfIDfromIfaceline()
 * @param iftype string type of subinterface (ATM,Serial ...)
 * @param ifaceline interface ID line see above
 */
void GIface::trimIfaceIDstrline(std::string& ifaceline,std::string iftype)
{
 if (iftype.empty()) {
  cout << __PRETTY_FUNCTION__ << "Error: empty iftype - interface config interface determinant " << endl;
  return;
 }
 string pattern="("+iftype+"[\\d:./]+)";
 Pattern* pPattern=Pattern::compile(pattern);
 Matcher* pMatcher=pPattern->createMatcher(ifaceline);
 if (pMatcher->findFirstMatch()) {
  //cout << "GIface::trimIfaceIDstrline - trimmed iface line : " << pMatcher->getGroup(1) << endl;
  ifaceline=pMatcher->getGroup(1);
 }
 delete pPattern;delete pMatcher;
}

//---------------------------------------------------------------------------
void GIface::printDebuginfo()
{
 cout << "GIface:" << getName() << endl;
 cout << "description:" << getDescr() << endl;
 cout << "GSubifaces:" << mSubIfVp.size() << endl;
 for(vector<GSubif*>::iterator iter=mSubIfVp.begin();iter<mSubIfVp.end();iter++)
 {
  cout << (*iter)->getName() << endl;
 }
}

//---------------------------------------------------------------------------
/**
 * add deep copy of GSubif pointer to our array of subifs
 * deep copy in needed, because parameter pointer can be deleled, and
 * also in most cases is deleted     
 */
void GIface::addCopySubifp(GSubif* pSubif)
{
 GSubif* pNewSubif=new GSubif(*pSubif);
 //cout << "pointer to Subif " << pNewSubif->getName() << " added in " << getName() << endl;
 mSubIfVp.push_back(pNewSubif);
}
