/*
* implementation of SonetController class
*/

//STDC++ and STL Headers
#include <iostream>
#include <sstream>
#include <stdexcept>

//C and local headers
#include "regexp/regexp/Matcher.h"
#include "regexp/regexp/Pattern.h"
#include "SonetController.h"
#include "E1Controller.h"
#include "Utils.h"

using namespace std;

//---------------------------------------------------------------------------
/**
 * normal constructor
 */
SonetController::SonetController()
{
 mpE1ControllerA=new E1Controller[mcNumSubE1];
 mType = eSonetContr; 
}

//---------------------------------------------------------------------------
/**
 * copy constructor, every class with "dynamic memory" members need copy
 * constructor
 */
SonetController::SonetController(const SonetController& m) : Controller(m)
{
 //create dynamic array
 mpE1ControllerA=new E1Controller[mcNumSubE1];
 for (int i=0;i<mcNumSubE1;i++) {
  mpE1ControllerA[i] = m.mpE1ControllerA[i];
 }
}

//---------------------------------------------------------------------------
/**
 * Destructor
 */
SonetController::~SonetController()
{
 delete [] mpE1ControllerA;
}

//---------------------------------------------------------------------------
/**
 * parse every line between SONET controller declaration and end  - ^! delimiter
 * 
 */
void SonetController::parsContrSection(std::string& rline)
{
 Pattern* pPattern;
 Matcher* pMatcher;

 bool ParseStop=false;
 
 //get controller ID
 pPattern=Pattern::compile("^controller SONET (\\S+)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  //cout << "controller ID line" << rline << " matched " << endl;
  string matchedstr=pMatcher->getGroup(1);
  //cout << "matched string " << matchedstr << endl;
  setName(matchedstr);
  ParseStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParseStop) return;

 //whole controller descriotion
 pPattern=Pattern::compile("^ description (.*)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  string matchedstr=pMatcher->getGroup(1);
  setDescr(matchedstr);
  ParseStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParseStop) return;

 //au-4 tug-3 section
 pPattern=Pattern::compile("au-4 (\\d+) tug-3 (\\d+)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  mAu4=Utils::strtoint(pMatcher->getGroup(1));
  mTug3=Utils::strtoint(pMatcher->getGroup(2));
  ParseStop=true;
 }
 delete pPattern;delete pMatcher;
 if (ParseStop) return;

 //sube1 section tug-2 e1 ... (description, channel-group etc)
 pPattern=Pattern::compile("tug-2 (\\d+) e1 (\\d+)");
 pMatcher = pPattern->createMatcher(rline);
 if (pMatcher->findFirstMatch()) {
  int Tug2=Utils::strtoint(pMatcher->getGroup(1));
  int E1=Utils::strtoint(pMatcher->getGroup(2));
  //sub E1 id between 1-63
  int SubE1num=getE1numId(mTug3,Tug2,E1);
  if (SubE1num>0 && SubE1num<(mcNumSubE1+1)) {
   //sub E1 index in array 0-62s
   E1Controller& rSubE1contr=mpE1ControllerA[SubE1num-1];
   //IMPORTANT: set controller name first, because from controller name is
   //derived other info (channel-group controller ID) in parsContrSection  
   if (rSubE1contr.getName().empty()) {
    rSubE1contr.setName("TU"+Utils::inttostr(SubE1num)+" "+getE1strId(mName,mAu4,mTug3,Tug2,E1));
   }
   rSubE1contr.parsContrSection(rline);
  }
  ParseStop=true;
 }
 delete pPattern;delete pMatcher;
}

unsigned int SonetController::getE1numId(unsigned int tug3, unsigned int tug2, unsigned int e1)
{
 return 21*(tug3-1)+3*(tug2-1)+e1;
} 

std::string SonetController::getE1strId(std::string SonetContrID,unsigned int au4,unsigned int tug3,unsigned int tug2, unsigned int e1)
{
 ostringstream osstrm;
 osstrm << SonetContrID << "." << au4 << "/" << tug3 << "/" << tug2 << "/" << e1;
 return osstrm.str();
} 

//---------------------------------------------------------------------------
/**
 * make links between channel groups in controller and corresponding serial interface 
 */
void SonetController::makeChangrpSerlinks(std::vector<GIface>& rSerIfacesV)
{
 for (int i=0;i<mcNumSubE1;i++) {
  E1Controller& rSubE1contr=mpE1ControllerA[i];
  rSubE1contr.makeChangrpSerlinks(rSerIfacesV);
 }
}

//---------------------------------------------------------------------------
void SonetController::printDebuginfo()
{
 cout << "Sonetcontroller " << mName << endl;
 for (int i=0;i<mcNumSubE1;i++) {
  E1Controller& rSubE1contr=mpE1ControllerA[i];
  rSubE1contr.printDebuginfo();
 }
}

//---------------------------------------------------------------------------
/**
 * get subE1 name for the concrete subE1 count number
 *
 * E.g. A whole sonet constroller has name 1/0, for 22 subE1, subid = 22, this
 * function returns 1/0.1/1/2/1/1
 * @throws out_of_range if subID is not in Sonet subid range
 */
std::string SonetController::getSubE1Name(unsigned int subE1id) const
{
 unsigned int tug3,tug2,e1;
 unsigned int modulo;
 
 if (subE1id>0 && subE1id<(mcNumSubE1+1)) {
  if (subE1id<22) tug3=1;
  else if (subE1id<43) tug3=2;
  else tug3=3;

  modulo=subE1id-(tug3-1)*21;
  tug2=modulo/3;

  if (modulo%3 > 0) {
   e1=modulo%3;
   tug2++;
  } else {
   e1=3;
  }
  
  return getE1strId(mName,1,tug3,tug2,e1);
 }
 throw out_of_range("subE1 in Sonet must be in interval <1-63>");
}
