/**
 * @file
 * @brief implementation of the SyncSummary class
 * @author Miroslav Kubelik
 * @date July 2007
 */

//STDC++ and STL Headers
#include <vector>
#include <iostream>

//C and local headers
#include "SyncSummary.h"
#include "E1Controller.h"
#include "E3Controller.h"
#include "SonetController.h"

using namespace std;

//---------------------------------------------------------------------------
/**
 * @brief constructor for a creating summary from the E1 controller information
 */
SyncSummary::SyncSummary(const E1Controller& e1Contr): mAllTimeslots(E1Controller::mcTimeslots),
  mFreeTimeslots(0),
  mAllE1s(1),
  mFreeE1s(0),
  mChannelGroups(0)
{
 //channel groups
 const std::vector<Changrp>& rChannelGroupsV = e1Contr.getAllChangr();

 //no channel groups, whole E1 is free
 if(rChannelGroupsV.size()==0) {
  addE1FreeSummary();
  return;
 }

 //unframed -  all timeslosts are used
 if(e1Contr.isUnframed()) {
  mFreeTimeslots = 0;
  mChannelGroups = 1;
  return;
 }

 unsigned int usedTS=0;
 //used timeslost through all E1s channel-groups
 for (vector<Changrp>::const_iterator chanGr=rChannelGroupsV.begin();chanGr<rChannelGroupsV.end();chanGr++) {
  usedTS+=chanGr->getUsedTSCount();
  mChannelGroups++;
 }

 //free = 32 (timeslots in E1) - used timeslots - 1
 //first timeslot (no. 0) in a framed E1 is for the framed synchronization
 mFreeTimeslots = E1Controller::mcTimeslots - usedTS - 1;
}

//---------------------------------------------------------------------------
/**
 * @brief constructor for a creating summary from the E3 controller information
 */
SyncSummary::SyncSummary(const E3Controller& e3Contr): mAllTimeslots(E1Controller::mcTimeslots * E3Controller::getSubE1Count()),
  mFreeTimeslots(0),
  mAllE1s(E3Controller::getSubE1Count()),
  mFreeE1s(0),
  mChannelGroups(0)
{
 /* const pointer to array of subE1 - see E3Controller constructor
  array is ALWAYS filled
 */
 const E1Controller* pE1A=e3Contr.getAllSubE1();
 int numE1count=e3Contr.getSubE1Count();
 for (int i=0;i<numE1count;i++) {
  const E1Controller& rE1contr=pE1A[i];
  //empty Name means that subE1 is not initialized in a configuration,
  //hence consider it as free E1
  if (rE1contr.getName().empty()) {
   addE1FreeSummary();
  } else {
   //calculate E1 Summary and add to te current result
   SyncSummary e1Sum(rE1contr);
   addFromSummary(e1Sum);
  }
 }
}

//---------------------------------------------------------------------------
/**
 * @brief constructor for a creating summary from the Sonet controller information
 */
SyncSummary::SyncSummary(const SonetController& sonetContr): mAllTimeslots(E1Controller::mcTimeslots * SonetController::getSubE1Count()),
  mFreeTimeslots(0),
  mAllE1s(SonetController::getSubE1Count()),
  mFreeE1s(0),
  mChannelGroups(0)
{
 /* const pointer to array of subE1 - see SonetController constructor
  array is ALWAYS filled
 */
 const E1Controller* pE1A=sonetContr.getAllSubE1();
 int numE1count=sonetContr.getSubE1Count();
 for (int i=0;i<numE1count;i++) {
  const E1Controller& rE1contr=pE1A[i];
  //empty Name means that subE1 is not initialized in a configuration,
  //hence consider it as free E1
  if (rE1contr.getName().empty()) {
   addE1FreeSummary();
  } else {
   //calculate E1 Summary and add to te current result
   SyncSummary e1Sum(rE1contr);
   addFromSummary(e1Sum);
  }
 }
}

//---------------------------------------------------------------------------
/**
 * Add 32 free timeslots and one E1 as free.
 */
void SyncSummary::addE1FreeSummary(bool addToAll)
{
 if (addToAll) {
   mAllTimeslots+=E1Controller::mcTimeslots;
   mAllE1s++;
 }
 mFreeTimeslots+=E1Controller::mcTimeslots;
 mFreeE1s++;
}

//---------------------------------------------------------------------------
/**
 * Add content of some other summary to the current summary.
 * @param summary summary, which summary properties will be added to our current
 * summary
 * @param addToAll if it is FALSE, "All" summary properties from the summary parameter
 * will not be added to the "All" properties in our summary, if it is TRUE
 * "All" summary properties from the summary parameter will be added
 * to the "All" properties in our summary, default is false
 */
void SyncSummary::addFromSummary(const SyncSummary& summary, bool addToAll)
{
 if (addToAll) {
   mAllTimeslots+=summary.mAllTimeslots;
   mAllE1s+=summary.mAllE1s;
 }
 mFreeTimeslots+=summary.mFreeTimeslots;
 mFreeE1s+=summary.mFreeE1s;
 mChannelGroups+=summary.mChannelGroups;
}

//---------------------------------------------------------------------------
/**
 * print info about summary - for debug
 */
void SyncSummary::printDebuginfo() const
{
 cout << "all timeslots: " << mAllTimeslots << endl;
 cout << "free timeslots: " << mFreeTimeslots << endl;
 cout << "all E1s: " << mAllE1s << endl;
 cout << "free E1s: " << mFreeE1s << endl;
 cout << "channel groups: " << mChannelGroups << endl;
}
