/**
 * @file
 * @brief the portinfo cisco device exporter implementation
 * @author Miroslav Kubelik
 * @date 2007
 */

//STDC++ and STL Headers
#include <string>
#include <sstream>
#include <iomanip>

//C and local headers
#include <time.h>
#include "CiscoDev.h"
#include "CiscoBox.h"
#include "CiscoE1Contr.h"
#include "CiscoE3Contr.h"
#include "CiscoSonetContr.h"
#include "CiscoBoxSummary.h"
#include "XMLSummaryConvertor.h"

using namespace std;

//------------------------------------------------------------------------------ 
/**
 * get XML for one Cisco device as XMLNode object
 * @param rCiscoBox cisco device object reference
 */
XMLNode Portinfo::CiscoDev::getXMLNode(const CiscoBox& rCiscoBox)
{
 //one device node
 XMLNode xDevNode=XMLNode::parseString( string("<router/>").c_str() );
 xDevNode.addAttribute("id",rCiscoBox.getID().c_str());
 xDevNode.addAttribute("hostname",rCiscoBox.getHostname().c_str());

 //add E1 controllers
 xDevNode.addChild(portinfoE1Controllers(rCiscoBox));
 //add E3 controllers
 xDevNode.addChild(portinfoE3Controllers(rCiscoBox));
 //add Sonet controllers
 xDevNode.addChild(portinfoSonetControllers(rCiscoBox));
 //add Summary
 xDevNode.addChild( getXMLSummary(rCiscoBox) );

 return xDevNode;
}

//------------------------------------------------------------------------------ 
/**
 * get XML LNI for one Cisco device as string with XML markup  
 */ 
std::string Portinfo::CiscoDev::getXMLexport(const CiscoBox& rCiscoBox)
{
 char* t=Portinfo::CiscoDev::getXMLNode(rCiscoBox).createXMLString();
 string retstr(t);
 free(t);
 return  retstr;
}

//------------------------------------------------------------------------------ 
/**
 * get all E1 controllers in one CiscoBox as the XML object
 */
XMLNode Portinfo::CiscoDev::portinfoE1Controllers(const CiscoBox& rCiscoBox)
{
 //const reference to vector of all E1 controllers
 const vector<E1Controller>& rE1ContrV=rCiscoBox.getAllE1();
 XMLNode xAllContrNode=XMLNode::parseString("<E1controllers/>");
 //iterator must be const
 for (vector<E1Controller>::const_iterator iter=rE1ContrV.begin();iter<rE1ContrV.end();iter++)
 {
  //E1 portinfo export
  Portinfo::CiscoE1Contr ExpE1Contr;
  xAllContrNode.addChild( ExpE1Contr.getXMLnode(*iter) );
 }
 return xAllContrNode;
}

//------------------------------------------------------------------------------
/**
 * get all E3 controllers in one CiscoBox as the XML object
 */
XMLNode Portinfo::CiscoDev::portinfoE3Controllers(const CiscoBox& rCiscoBox)
{
 //const reference to vector of all E3 controllers
 const vector<E3Controller>& rE3ContrV=rCiscoBox.getAllE3();
 XMLNode xAllContrNode=XMLNode::parseString("<E3controllers/>");
 //iterator must be const
 for (vector<E3Controller>::const_iterator iter=rE3ContrV.begin();iter<rE3ContrV.end();iter++)
 {
  //E3 portinfo export
  Portinfo::CiscoE3Contr ExpE3Contr;
  xAllContrNode.addChild( ExpE3Contr.getXMLnode(*iter) );
 }
 return xAllContrNode;
}

//------------------------------------------------------------------------------
/**
 * get all SONET controllers in one CiscoBox as the XML object
 */
XMLNode Portinfo::CiscoDev::portinfoSonetControllers(const CiscoBox& rCiscoBox)
{
 //const reference to vector of all E3 controllers
 const vector<SonetController>& rSonetContrV=rCiscoBox.getAllSonet();
 XMLNode xAllContrNode=XMLNode::parseString("<Sonetcontrollers/>");
 //iterator must be const
 for (vector<SonetController>::const_iterator iter=rSonetContrV.begin();iter<rSonetContrV.end();iter++)
 {
  //E3 portinfo export
  Portinfo::CiscoSonetContr ExpSonetContr;
  xAllContrNode.addChild( ExpSonetContr.getXMLnode(*iter) );
 }
 return xAllContrNode;
}

//------------------------------------------------------------------------------
/**
 * get summary for whole CiscoBox as the XML object
 */
XMLNode Portinfo::CiscoDev::getXMLSummary(const CiscoBox& rCiscoBox)
{
 //create summary
 CiscoBoxSummary summary(rCiscoBox);

 return XMLSummaryConvertor::getXMLCiscoBoxSummary(summary);
}
