/**
 * @file
 * @brief an implementation for a XML convertor for the summary classes
 * @author Miroslav Kubelik
 * @date 2007
 */

//STDC++ and STL Headers
#include <sstream>
#include <iomanip>

//C and local headers
#include "XMLSummaryConvertor.h"
#include "CiscoBoxSummary.h"
#include "SyncSummary.h"
#include "Utils.h"

using namespace std;

//------------------------------------------------------------------------------
/**
 * get a portinfo XML object for one cisco device summary
 * @param summary cisco device summary object
 * @param sumName name for the summary element in the output
 */
XMLNode Portinfo::XMLSummaryConvertor::getXMLCiscoBoxSummary(const CiscoBoxSummary& summary,const std::string& sumName)
{
 XMLNode xSummary=XMLNode::parseString( string("<"+sumName+"/>").c_str() );
 //E1 summary
 xSummary.addChild( getXMLSyncSummary(summary.getE1Summary(),"E1summary") );
 //E3 summary
 xSummary.addChild( getXMLSyncSummary(summary.getE3Summary(),"E3summary") );
 //Sonet summary
 xSummary.addChild( getXMLSyncSummary(summary.getSonetSummary(),"SONETsummary") );
 //All Sync Summary
 xSummary.addChild( getXMLSyncSummary(summary.getWholeSyncSummary(),"Syncsummary") );

 return xSummary;
}

//------------------------------------------------------------------------------
/**
 * get a portinfo XML object for one synchronous summary
 * @param sum synchronous summary object
 * @param sumName name for the summary element in the output
 */
XMLNode Portinfo::XMLSummaryConvertor::getXMLSyncSummary(const SyncSummary& sum,const std::string& sumName)
{
 float util;

 XMLNode xSummary=XMLNode::parseString( string("<"+sumName+"/>").c_str() );

 //all timeslots summary
 XMLNode xTimeslotsNode = xSummary.addChild("timeslosts");
 XMLNode xSubNode=xTimeslotsNode.addChild("summary");
 xSubNode.addText(  Utils::inttostr(sum.getAllTimeslots()).c_str() );
 xSubNode=xTimeslotsNode.addChild("used");
 xSubNode.addText(  Utils::inttostr(sum.getUsedTimeslots()).c_str() );
 xSubNode=xTimeslotsNode.addChild("free");
 xSubNode.addText(  Utils::inttostr(sum.getFreeTimeslots()).c_str() );

 //timeslots utilization
 xSubNode=xTimeslotsNode.addChild("utilization");
 if (sum.getAllTimeslots()>0) {
  util = Utils::getUtilization(sum.getAllTimeslots(),sum.getUsedTimeslots());
  xSubNode.addText( formatUtilization(util).c_str() );
 } else {
  xSubNode.addText( "undef" );
 }

 //all E1 Summary
 XMLNode xE1sNode = xSummary.addChild("E1s");
 xSubNode=xE1sNode.addChild("summary");
 xSubNode.addText(  Utils::inttostr(sum.getAllE1()).c_str() );
 xSubNode=xE1sNode.addChild("used");
 xSubNode.addText(  Utils::inttostr(sum.getUsedE1()).c_str() );
 xSubNode=xE1sNode.addChild("free");
 xSubNode.addText(  Utils::inttostr(sum.getFreeE1()).c_str() );

 //E1 utilization
 xSubNode=xE1sNode.addChild("utilization");
 if (sum.getAllE1()>0) {
  util = Utils::getUtilization(sum.getAllE1(),sum.getUsedE1());
  xSubNode.addText( formatUtilization(util).c_str() );
 } else {
  xSubNode.addText("undef");
 }
 
 //count of the channel groups
 xSubNode=xSummary.addChild("channelgroups");
 xSubNode.addText(  Utils::inttostr(sum.getChannelGroups()).c_str() );

 return xSummary;
}

//------------------------------------------------------------------------------
/**
 * get the utilization as formatted string
 */
std::string Portinfo::XMLSummaryConvertor::formatUtilization(float val)
{
 ostringstream utilStrm;
 utilStrm << setprecision(4) << val;
 return utilStrm.str();
}
