/**
 * @file
 * @ingroup Programs
 * @brief module with the RRD functions for portinfo exports - implementations
 * @author Miroslav Kubelk
 * @date 2007
 */

//STDC++ and STL Headers
#include <vector>
#include <sstream>
#include <stdexcept>

//C and local headers
#include "portinfoRRD.h"
#include "rrd/RRDWrap.h"
#include "SyncSummary.h"
#include "Utils.h"

using namespace std;

//------------------------------------------------------------------------------
/**
 * Creates the rrd file
 */
void createRRD(const std::string& fileName)
{
  RRDWrap rrdWr(fileName);

  //arguments
  vector<string> argsV;
  argsV.push_back("--step");argsV.push_back("86400");
  argsV.push_back("--start");argsV.push_back("-31536000");
  argsV.push_back("DS:usedts:GAUGE:172800:U:U");
  argsV.push_back("DS:freets:GAUGE:172800:U:U");
  argsV.push_back("DS:usede1:GAUGE:172800:U:U");
  argsV.push_back("DS:freee1:GAUGE:172800:U:U");
  argsV.push_back("RRA:LAST:0.5:1:400");

  rrdWr.create(argsV);
}

//------------------------------------------------------------------------------
/**
 * updates the rrd file
 */
void updateRRD(const std::string& fileName,const SyncSummary& sum)
{
  RRDWrap rrdWr(fileName);

  ostringstream osstrm;
  osstrm << "N:"
   << sum.getUsedTimeslots() << ":"
   << sum.getFreeTimeslots() << ":"
   << sum.getUsedE1() << ":"
   << sum.getFreeE1();

  rrdWr.update(osstrm.str());
}

//------------------------------------------------------------------------------
/**
 * create graphs from the RRD database for the portinfo report
 */
void createGraphRRD(const std::string& exportName,const std::string& mediaDir)
{
  //helper stream
  ostringstream osstrm;

  string rrdName( ( (!mediaDir.empty())? mediaDir+'/' : "" )+exportName+".rrd");
  string tsGraphName( ( (!mediaDir.empty())? mediaDir+'/' : "" )+exportName+"ts.png");
  string e1GraphName( ( (!mediaDir.empty())? mediaDir+'/' : "" )+exportName+"e1.png");

  //localtime
  time_t rawtime;
  time ( &rawtime );
  string timestr(ctime (&rawtime));
  Utils::trim(timestr,"\n");
  
  RRDWrap rrdWr(rrdName);

  //arguments
  vector<string> argsV;
  
  //timeslots graph
  argsV.push_back(tsGraphName);
  argsV.push_back("--units-exponent");argsV.push_back("0");
  argsV.push_back("--lower-limit");argsV.push_back("0");
  argsV.push_back("--alt-autoscale-max");
  argsV.push_back("--alt-y-grid");
  argsV.push_back("--start");argsV.push_back("-31536000");
  argsV.push_back("--vertical-label");argsV.push_back(exportName);
  argsV.push_back("--title");
  //title
  osstrm.str("");
  osstrm << "Timeslots statistics " << "(" << timestr << ")";
  argsV.push_back( osstrm.str() );
  argsV.push_back("DEF:usedtimeslots="+rrdName+":usedts:LAST");
  argsV.push_back("DEF:freetimeslots="+rrdName+":freets:LAST");
  argsV.push_back("CDEF:alltimeslots=usedtimeslots,freetimeslots,+");
  argsV.push_back("AREA:usedtimeslots#FF3030:Used");
  argsV.push_back("GPRINT:usedtimeslots:LAST:%1.0lf timeslots.");
  argsV.push_back("STACK:freetimeslots#00CC00:Free");
  argsV.push_back("GPRINT:freetimeslots:LAST:%1.0lf timeslots.");
  argsV.push_back("LINE1:alltimeslots#000001:All");
  argsV.push_back("GPRINT:alltimeslots:LAST:%1.0lf timeslots.");

  rrdWr.graph(argsV);

  argsV.clear();

  //E1 graph
  argsV.push_back(e1GraphName);
  argsV.push_back("--units-exponent");argsV.push_back("0");
  argsV.push_back("--lower-limit");argsV.push_back("0");
  argsV.push_back("--alt-autoscale-max");
  argsV.push_back("--alt-y-grid");
  argsV.push_back("--start");argsV.push_back("-31536000");
  argsV.push_back("--vertical-label");argsV.push_back(exportName);
  argsV.push_back("--title");
  //title
  osstrm.str("");
  osstrm << "E1 port statistics " << "(" << timestr << ")";
  argsV.push_back( osstrm.str() );
  argsV.push_back("DEF:usede1s="+rrdName+":usede1:LAST");
  argsV.push_back("DEF:freee1s="+rrdName+":freee1:LAST");
  argsV.push_back("CDEF:alle1s=usede1s,freee1s,+");
  argsV.push_back("AREA:usede1s#BEB08E:Used");
  argsV.push_back("GPRINT:usede1s:LAST:%1.0lf E1\'s.");
  argsV.push_back("STACK:freee1s#3096FC:Free");
  argsV.push_back("GPRINT:freee1s:LAST:%1.0lf E1\'s.");
  argsV.push_back("LINE1:alle1s#000001:All");
  argsV.push_back("GPRINT:alle1s:LAST:%1.0lf E1\'s.");

  rrdWr.graph(argsV);
}
