/**
 * @file
 * @ingroup Programs
 * @brief portinfo XML export for cisco devices - implementation
 * @author Miroslav Kubelk
 * @date 2007
 *
 * Main program for the making of the XML exports. This program has two
 * using options
 * - as XML exporter for a group of devices which are defined as command line
 *   parameters
 * - the definition for the exports is in the XML file
 */

//STDC++ and STL Headers
#include <iostream>
#include <fstream>
#include <vector>
#include <sstream>
#include <stdexcept>

/* config from the autoconf */
#include "config.h"

//C and local headers
#include "portinfoexp.h"
#include "CiscoBox.h"
#include "CiscoconfParser.h"
#include "portinfo/CiscoDev.h"
#include "portinfo/CiscoDevGroup.h"
#include "Utils.h"
#include "PortinfoXMLConfig.h"

#if RRD_SUPPORT == 1
 #include "portinfoRRD.h"
#endif

//command line arguments handler
#include "portinfoexpopt.h"

//global variables - for this module
/** a whole XML report name */
std::string gReportName;
/** path to the directory with the device configurations */
std::string gDeviceConfDir;
/** vector for the devices */
std::vector<std::string> gdevicesV;
/** the directory for media files - images, rrd files etc. */
std::string gmediaDir;
/** the xml Configuration object */
PortinfoXMLConfig gxmlConfig;
/** if we want to process devices along the XML configuration */
bool gXmlConfGiven=false;

//forward function declaration - some functions are defined in header files from other modules
//maybe better approach is use header file for the definitions, but these functions
//are used only in this module, so I think that this local declaration is better
bool checkCmdArg(int argc, char **argv);
std::string getDevFilename(const std::string& devName,const std::string& confDirName);
void processByXml();
std::string getFullMediaFileName(const std::string& reportName,const std::string& mediaDir,const std::string& extPart);
void saveXMLgroupExport(const std::string& reportName,const std::string& exportDir,const Portinfo::CiscoDevGroup& devGroup);
void fillExportGroup(Portinfo::CiscoDevGroup& devGroup,const std::string& confDir, const std::vector<DeviceValObj>& devValObjV);
void fillValueObjVector(const PortinfoXMLConfig::groupT& group, std::vector<DeviceValObj>& devValObjV);
void makeEveryDeviceExport(const std::vector<DeviceValObj>& devValObjV,const std::string& confDir,const std::string& exportDir);
void makeRRDexport(const std::string& reportName,const std::string& mediaDir,const SyncSummary& sum);

using namespace std;

//------------------------------------------------------------------------------
/**
 * main entry point
 */ 
int main(int argc, char* argv[])
{
 //handle command line arguments
 if (!checkCmdArg(argc,argv)) {
  return(EXIT_FAILURE);
 }

 //process devices along the XML configuration
 if (gXmlConfGiven) {
  try {
   ::processByXml();
  } catch (const exception& e) {
   cerr << "Got an exception: " << e.what() << endl;
   exit (EXIT_FAILURE);
  }
  return(EXIT_SUCCESS);
 }
 
 //input file
 ifstream inFileStrm;

 try {
  //create device group report
  //portinfo device group
  Portinfo::CiscoDevGroup devGroup;

  //set a group name, this is a report name in the XML export
  devGroup.setName(gReportName);

  //vector of device value object for all devices from the commad line
  vector<DeviceValObj> devicesValOV;
  //fill devicesValOV vector
  for (vector<string>::const_iterator iter=gdevicesV.begin();iter<gdevicesV.end();iter++) {
   //temporary valu object
   DeviceValObj tempObj;
   //set id,aipAddr as device name from the command line
   tempObj.id=*iter;
   tempObj.ipAddr=*iter;
   //add copy to the vector
   devicesValOV.push_back(tempObj);
  }

  //make export for the group
  ::fillExportGroup(devGroup,gDeviceConfDir,devicesValOV);

  //make a group summary
  devGroup.createSummary();

  cout << devGroup.getXMLexport() << endl;

  #if RRD_SUPPORT == 1
   ::makeRRDexport(gReportName,gmediaDir, devGroup.getGroupSummary().getWholeSyncSummary() );
  #endif

 } catch (const exception& e) {
  cerr << "Got an exception: " << e.what() << endl;
  exit (EXIT_FAILURE);
 }

 return(EXIT_SUCCESS);
}

//------------------------------------------------------------------------------
/**
 * get a full path to the device configuration file
 * @param devName a device name
 * @param confDirName a name for the directory with configurations
 */
std::string getDevFilename(const std::string& devName,const std::string& confDirName)
{
 return confDirName+"/"+devName;
}

//------------------------------------------------------------------------------
/**
 * Get a file name with directory path for all media files (pictures, rrd)
 * @param reportName name for the report
 * @param mediaDir directory for the media files
 * @param extPart string wich will be copiied at the end of the base name string
 *
 * Example: reportname="testReport", mediaDir="/usr/local/media", extPart="Timeslot.rrd"
 * final string will be /usr/local/media/testReportTimeslot.rrd
 */
std::string getFullMediaFileName(const std::string& reportName,const std::string& mediaDir,const std::string& extPart)
{
 ostringstream osstrm;

 if (!mediaDir.empty()) {
  osstrm << mediaDir << '/';
 }

 osstrm << reportName;
 osstrm << extPart;

 return osstrm.str();
}

//------------------------------------------------------------------------------
/** Process devices by along XML file configuration.
 *  It makes both all XML devices exports and XML group exports
 * @throw exception when an export process fails
 */
void processByXml()
{
 //export directory
 string exportDir=gxmlConfig.getReportDir();
 //media directory
 string mediaDir=gxmlConfig.getMediaDir();
 //the directorty with the device configuration
 string deviceConfDir=gxmlConfig.getDevConfDir();
 //vector of device values objects, is used for group the export generation
 vector<DeviceValObj> devicesValOV;

 //get all devices
 const vector<PortinfoXMLConfig::deviceT>& allDevices = gxmlConfig.getDevices();
 //fill devicesValOV vector
 for (vector<PortinfoXMLConfig::deviceT>::const_iterator devIter=allDevices.begin();devIter<allDevices.end();devIter++) {
  //temporary object
  DeviceValObj tempObj;
  //set properties by XML configuration
  tempObj.id=devIter->id;
  tempObj.ipAddr=devIter->ipAddr;
  //add copy to the vector
  devicesValOV.push_back(tempObj);
 }

 //save export for every device
 ::makeEveryDeviceExport(devicesValOV,deviceConfDir,exportDir);


 //all device groups
 const vector<PortinfoXMLConfig::groupT>& xmlGroups=gxmlConfig.getGroups();

 //process devices groups from a XML configuration
 for (vector<PortinfoXMLConfig::groupT>::const_iterator xmlGrIter=xmlGroups.begin();xmlGrIter<xmlGroups.end();xmlGrIter++) {
  //create device group report
  //portinfo device group
  Portinfo::CiscoDevGroup portInfoDevGroup;
  
  cout << "make the export for the group " << xmlGrIter->name << endl;

  //set a group name, this is a report name in the XML export
  portInfoDevGroup.setName(xmlGrIter->report);

  //clear vector of device value object for one XML device group
  devicesValOV.clear();
  ::fillValueObjVector(*xmlGrIter,devicesValOV);

  //make export for the group
  ::fillExportGroup(portInfoDevGroup,deviceConfDir,devicesValOV);

  //make group summary
  portInfoDevGroup.createSummary();

  //save a portinfo group and summary
  ::saveXMLgroupExport(xmlGrIter->report,exportDir,portInfoDevGroup);

  #if RRD_SUPPORT == 1
   ::makeRRDexport(xmlGrIter->report,mediaDir, portInfoDevGroup.getGroupSummary().getWholeSyncSummary() );
  #endif
 }//end for vector<groupT>
}

//------------------------------------------------------------------------------
/**
 * save XML export for a portinfo device group to the XML file
 *
 * @param reportName name for the XML report, must be used without .xml extension
 * @param exportDir directory for the XML export
 * @param devGroup portinfo device group object
 *
 * @throw runtime_error when IO failed
 */
void saveXMLgroupExport(const std::string& reportName,const std::string& exportDir,const Portinfo::CiscoDevGroup& devGroup)
{
 ofstream fout;

 string fullXmlFileName( ( (!exportDir.empty())? exportDir+'/' : "" )+reportName+".xml" );
 fout.open(fullXmlFileName.c_str());
 while(fout.good() == false) throw runtime_error("Error: Unable to open file for the group export - "+fullXmlFileName+" for writing");

 fout << devGroup.getXMLexport() << endl;
 fout.close();
}

//------------------------------------------------------------------------------
/**
 * Make RRD export for a synchronous summary.
 *
 * @param reportName a name for a rrd file without .rrd extension
 * @param mediaDir a directory where will be rrd file saved
 * @throw runtime_exception when rrd support isn't presented
 */
void makeRRDexport(const std::string& reportName,const std::string& mediaDir,const SyncSummary& sum)
{
#if RRD_SUPPORT == 1
 string rrdFileName = ::getFullMediaFileName(reportName,mediaDir,".rrd");
 //create the rrd file if it doesn't exist
 if (!Utils::FileExist(rrdFileName)) {
  createRRD(rrdFileName);
 }
 //update rrd file
 updateRRD(rrdFileName,sum);
 //generate graphs
 createGraphRRD(reportName,mediaDir);
#else
 throw new runtime_error("Hey, what do you make? RRD support isn't compiled!");
#endif
}

//------------------------------------------------------------------------------
/**
 * Fills Portinfo::CiscoDevGroup object with the portinfo export for the
 * group of device defined by vector of the DeviceValObj objects
 *
 * @param devGroup the export group object which will be filled
 * @param confDir the path for derectory with device configurations
 * @param devValObjV the vector with devices for which will be a group export created
 * @throw runtime_error when a reading from the configuration file fails
 */
void fillExportGroup(Portinfo::CiscoDevGroup& devGroup,const std::string& confDir,
 const std::vector<DeviceValObj>& devValObjV)
{
 //input file
 ifstream inFileStrm;

 for (vector<DeviceValObj>::const_iterator iter=devValObjV.begin();iter<devValObjV.end();iter++) {
  //alias for the iter->ipAddr, much better variable name
  const string& deviceName = iter->ipAddr;

  //if cannot open a file with the device configuration get an exception
  string devFileName=::getDevFilename(deviceName,confDir);
  inFileStrm.open(devFileName.c_str());
  while(inFileStrm.good() == false) throw runtime_error("Error: Unable to open file "+devFileName);

  //configuration parser
  CiscoconfParser* pParser=new CiscoconfParser();
  pParser->parse(inFileStrm);

  //close and clear the file stream
  inFileStrm.close();
  inFileStrm.clear();

  CiscoBox* pCiscoBox=pParser->getCiscoBox();
  //make connections between serial links and channel groups, it's necessary
  pCiscoBox->makeChangrpSerlinks();
  pCiscoBox->setHostname(deviceName);
  pCiscoBox->setID(iter->id);

  //add copy of the cisco device to the portinfo device group
  devGroup.addCiscoBox(*pCiscoBox);
  //free parser and ciscobox objects also
  delete pParser;
 } //end for vector of device names
}

//------------------------------------------------------------------------------
/**
 * Fills vector of device value objects by the content of group from the XML
 * configuration
 *
 * @param group the configuration group object
 * @param devValObjV the vector which will be filled
 */
void fillValueObjVector(const PortinfoXMLConfig::groupT& group, std::vector<DeviceValObj>& devValObjV)
{
 for(vector<PortinfoXMLConfig::deviceT>::const_iterator devIter=group.members.begin(); devIter<group.members.end(); devIter++) {
  //temporary value object
  DeviceValObj tempObj;
  tempObj.id = devIter->id;
  tempObj.ipAddr = devIter->ipAddr;
  //add copy of tempory value object to the vector
  devValObjV.push_back(tempObj);
 }
}

//------------------------------------------------------------------------------
/**
 * Make XML export for every device from a vector (array) of devices
 *
 * @param devValObjV a vector with device value objects for the export
 * @param confDir a directory with the configuration files for every device
 * @param exportDir a directory where will be the exports saved
 * @throw exception which can bubbles from the called code for a saving exported files or
 * friom code for a making an XML export.
 */
void makeEveryDeviceExport(const std::vector<DeviceValObj>& devValObjV,const std::string& confDir,
const std::string& exportDir)
{
 for (vector<DeviceValObj>::const_iterator iter=devValObjV.begin();iter<devValObjV.end();iter++) {

  cout << "make the export for the device " << iter->id << endl;

  //export group
  Portinfo::CiscoDevGroup devGroup;
  //set an export name - device id
  devGroup.setName(iter->id);

  //vector of device value objects, it will be filled only with a current device
  //because we need export only for one device
  vector<DeviceValObj> exportV(1,*iter);

  //make export
  ::fillExportGroup(devGroup,confDir,exportV);

  //make a group summary
  devGroup.createSummary();

  ::saveXMLgroupExport(devGroup.getName(),exportDir,devGroup);
  
  #if RRD_SUPPORT == 1
   ::makeRRDexport(devGroup.getName(),exportDir, devGroup.getGroupSummary().getWholeSyncSummary() );
  #endif
 }
}

//------------------------------------------------------------------------------
/**
 * Check the command line arguments and fill global variables from this
 * arguments.
 * @return true if all necessary arguments and its parameters are correctly
 *  set, false otherwise
 */
bool checkCmdArg(int argc, char **argv)
{
 //structure for cmdline argument
 gengetopt_args_info args_info;

 /*call our cmdline parser */
 if (cmdline_parser (argc, argv, &args_info) != 0) exit(1) ;

 //print version
 if (args_info.version_given) {
  cout << CMDLINE_PARSER_PACKAGE << " " << CMDLINE_PARSER_VERSION << endl;
  cout <<  "Written (C) 2007 Miroslav Kubelik - koubel@volny.cz" << endl;
  cout <<  "This program is released under the GNU General Public License." << endl;
  exit(EXIT_SUCCESS);
 }

 //the xml configuration file is presented, hence ignore other options
 if (args_info.xmlconf_given) {
  try {
   gxmlConfig.readFromFile(args_info.xmlconf_arg);
   gXmlConfGiven=true;
  } catch (const exception& e) {
   cmdline_parser_free (&args_info);
   cerr << "Exception in-process of the XML config reading: " << e.what() << endl;
   exit (EXIT_FAILURE);
  }
  cmdline_parser_free (&args_info);
  return true;
 }
 
 //other options, when the xml config isn't presented
 if (args_info.devconfdir_given) {
  gDeviceConfDir=args_info.devconfdir_arg;
 } else {
  cerr << "No xml config given." << endl;
  cerr << "'--devconfdir' ('-d') option required" << endl;
  cmdline_parser_free (&args_info);
  return false;
 }
 if (args_info.reportname_given) {
  gReportName=args_info.reportname_arg;
 } else {
  cerr << "No xml config given." << endl;
  cerr << "'--reportname' ('-r') option required" << endl;
  cmdline_parser_free (&args_info);
  return false;
 }
 if (args_info.mediadir_given) {
  gmediaDir=args_info.mediadir_arg;
 } else {
  gmediaDir="";
 }

 //unnamed options - arguments
 if (args_info.inputs_num<=0) {
  cout << "No desired device name is specified!" << endl;
  cmdline_parser_free (&args_info);
  return false;
 }
 
 //store device names parameters in the vector
 for ( unsigned i = 0 ; i < args_info.inputs_num ; ++i ) {
  gdevicesV.push_back ( args_info.inputs[i] );
 }

 cmdline_parser_free (&args_info);
 return true;
}
