/**
 * @file
 * @brief util class implementation
 * @author Miroslav Kubelik
 * @date April 2007
 */

/* autoheader config */
#include "../src/config.h"

//STDC++ and STL Headers
#include <string>
#include <sstream>
#include <fstream>
#include <stdexcept>

//C and local headers
#include <assert.h>
#include <dirent.h>
#include "regexp/regexp/Matcher.h"
#include "regexp/regexp/Pattern.h"
#include "Utils.h"

#ifndef HAVE_SCANDIR
 #include "scandir.h"
#endif

using namespace std;

//---------------------------------------------------------------------------
/*
 * simple strtoint C++ conversion
 */
int Utils::strtoint(std::string numbstr)
{
istringstream istr(numbstr);
unsigned int nb=0;

 istr >> nb;
 return nb;
}

//---------------------------------------------------------------------------
/*
 * simple inttostr C++ conversion
 */
std::string Utils::inttostr(int num)
{
 ostringstream o;
 if (o << num) {
  return o.str();
 } else {
  return "-88888888888888";
 }
}

//---------------------------------------------------------------------------
/*
 * convert string to uppercase
 */
std::string Utils::upperCase(std::string& s) {
  char* buf = new char[s.length()];
  s.copy(buf, s.length());
  for(int i = 0; i < (int)s.length(); i++)
    buf[i] = toupper(buf[i]);
  string r(buf, s.length());
  delete buf;
  return r;
}

//---------------------------------------------------------------------------
/*
 * is string first match regexp pattern, return true or false
 */
bool Utils::regFirstMatch(std::string pattern, std::string text) {
 Pattern* pPattern=NULL;
 Matcher* pMatcher=NULL;

 /* IMPORTANT, we must use this variable for save returned value,
    otherwise we have memory leaks
 */
 bool retval=false;

 pPattern = Pattern::compile(pattern);
 pMatcher = pPattern->createMatcher(text);
 if (pMatcher->findFirstMatch()) {
  retval = true;
 }
 delete pPattern;delete pMatcher;  
 return retval;
}

//---------------------------------------------------------------------------
/**
 * trim function by Marco Dorantes
 * from http://blogs.msdn.com/marcod/archive/2005/07/08/StdTrim.aspx
 */
std::string Utils::trim(std::string& s,const std::string& drop)
{
 string r=s.erase(s.find_last_not_of(drop)+1);
 return r.erase(0,r.find_first_not_of(drop));
}

//---------------------------------------------------------------------------
/**
 * read whole file to string and return it
 */
std::string Utils::ReadFile(const std::string& filename_i)
{
// --- Using getline() ---
string line, str;
ifstream infile (filename_i.c_str());
  assert (infile.is_open());

  while (getline (infile, line))
  {
    str.append(line);
    str.append("\n");
    line.erase();
  }

  return str; 
}

//---------------------------------------------------------------------------
/**
 * check if file exist
 */
bool Utils::FileExist(std::string FileName)
{
 ifstream FileL;
 FileL.open(FileName.c_str());
 if(!FileL) {
  FileL.close();
  return false;
 }  else {
  FileL.close();
  return true;
 }
}

//---------------------------------------------------------------------------
/** check if string is in string vector
 * @return bool
 */
bool Utils::inStringVector(const std::string& str, const std::vector<std::string>& vect)
{
 return inVector<string>(str,vect);
}

//---------------------------------------------------------------------------
/** add quotes around string */
void Utils::addQuotes(std::string& str)
{
 ostringstream strm;

 strm << "\"" << str << "\"";
 str = strm.str();
}

//---------------------------------------------------------------------------
/** add quotes around string */
std::vector<std::string> Utils::getDirFiles(std::string& dirname,bool filterDot)
{
 struct dirent **nameList;
 std::vector<std::string> fnames;
 
 //scan directory
 int numberOfFiles = scandir(dirname.c_str(),&nameList,0,0);
 if (numberOfFiles < 1)
 {
  free(nameList);
  return fnames;
 }

 //go through all files
 while (numberOfFiles--) {
  //filter dot (. and ..) if necessary
  if (filterDot &&
       ( strcmp(nameList[numberOfFiles]->d_name, ".")==0 ||
         strcmp(nameList[numberOfFiles]->d_name, "..")==0
       )
     )
  {
   free(nameList[numberOfFiles]);
   break;
  }
  fnames.push_back(nameList[numberOfFiles]->d_name);
  free(nameList[numberOfFiles]);
 }
 free(nameList);
 return fnames;
}

//---------------------------------------------------------------------------
/** tokenizer */
void Utils::Tokenize(const std::string& str,
              std::vector<std::string>& tokens,
              const std::string& delimiters)
{
    // Skip delimiters at beginning.
    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    string::size_type pos     = str.find_first_of(delimiters, lastPos);

    while (string::npos != pos || string::npos != lastPos)
    {
        // Found a token, add it to the vector.
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}

//------------------------------------------------------------------------------
/**
 * Get a percentage utilization of some values
 * @param wholeCount whole count
 * @param valCount value count
 * @return valCount/wholeCount * 100
 * @throws out_of_range if wholeCount is equal to zero
 */
float Utils::getUtilization(unsigned int wholeCount,unsigned int valCount)
{
 if (wholeCount == 0) throw out_of_range("the exception occures when calc utilization, the wholeCount is zero");
 return ( valCount/(float)wholeCount * 100);
}
