<?php
/**
 * Implementation of Device Controller actions.
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Serves Device Controller actions.
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class DeviceController extends Zend_Controller_Action
{
//==============================================================================
/** Sets main menu into the "Devices" item.
 */
public function init()
{
 SiteTemplatePlugin::setMainMenu(SiteTemplatePlugin::MAINMENU_DEVICES);
}

//==============================================================================
/**
 * Action handler for index page. A view shows list of devices with a pagination
 * if necessary.
 * @return void
 */
public function indexAction()
{
 //left block
 $this->view->leftBlocks = array(LeftBlock_SimpleSearch::getDeviceSearch());

 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;

 //device URL enumerator
 $devUrlEnum=new DeviceUrlEnum();
 //device object
 $device=new CiscoDevice();
 $deviceCount=$device->getAllDevicesCount();
 
 //page navigator
 $pgNav=new PageNav($pageNo,$deviceCount);
 $pgNav->MainUrl=$devUrlEnum->index;

 //devices in the current page
 $this->view->devices=$device->getAllDevices($pageNo,$pgNav->getPerPageItems(),true);
 //all devices count
 $this->view->deviceCount=$deviceCount;
 //device url enumerator for a  view
 $this->view->devUrlEnum=$devUrlEnum;
 //add page navigator XHTML code for a view
 $this->view->pageNav=$pgNav->getNavXhtmlContent();
}

//==============================================================================
/**
 * Action handler for the detail view information about one device.
 * @return void
 */
public function viewAction()
{
 //object from the parameters
 $device=new CiscoDevice();
 $device->setDataById($this->_getParam("device"));

 //device url enumerator
 $devUrlEnum=DeviceUrlEnum::factory($device);

 //infinite bidirectional itarator over all devices
 $iter = new BidirInfSimpleArrIterator($device->getAllDevices());
 //seek to current item
 $iter->seek($device);
 //next item
 $iter->next();$nextItem=$iter->current();
 $devUrlEnum->setFromDevice($nextItem);
 $this->view->nextItem=$nextItem;
 $this->view->nextItemUrl=$devUrlEnum->detail;
 //go to previous = twice back
 $iter->previous();$iter->previous();
 //previous
 $prevItem=$iter->current();
 $devUrlEnum->setFromDevice($prevItem);
 $this->view->prevItem=$prevItem;
 $this->view->prevItemUrl=$devUrlEnum->detail;
 
 //left block
 $this->view->leftBlocks = array(LeftBlock_SimpleSearch::getDeviceSearch());

 //device properties for a view
 $this->view->devId=$device->Id;
 $this->view->devIPaddr=$device->IpAddr;
 $this->view->reportCreateDate=$device->CreatedDT->format("r");
 $devUrlEnum->setFromDevice($device);
 $this->view->devUrlEnum=$devUrlEnum;

 //e1 controllers
 $count=$device->getAllE1ControllersCount();
 $this->view->e1ContrCount=$count;
 if ($count>0) $this->view->e1Controllers=$device->getAllE1Controllers();

 //e3 controllers
 $count=$device->getAllE3ControllersCount();
 $this->view->e3ContrCount=$count;
 if ($count>0) $this->view->e3Controllers=$device->getAllE3Controllers();

 //sonet controllers
 $count=$device->getAllSonetControllersCount();
 $this->view->sonetContrCount=$count;
 if ($count>0) $this->view->sonetControllers=$device->getAllSonetControllers();

 //summary
 $this->view->deviceSummary=$device->Summary;
}

//==============================================================================
/**
 * Action handler for the search results, action serves request from the simple
 * search dialog and advanced search dialog if any exists.
 * @return void
 */
public function searchResultAction()
{
 //device instance
 $device=new CiscoDevice();
 //device url enumerator
 $devUrlEnum=DeviceUrlEnum::factory($device);

 //search object
 $search=Search::factorySearch("SearchDevice");
 //get current page number, default is 1
 $pageNo=$this->_hasParam(PageNav::NAVPARAM) ? $this->_getParam(PageNav::NAVPARAM) : 1;
 //set pagination in the search
 $search->setPageParams($pageNo,PageNav::PER_PAGE_COUNT);

 //if reguest is from simple search dialog ("simpleSearch" index is presented)
 //replace "searchText" index by "id" index
 if ( array_key_exists("simpleSearch",$params=Utils::getHTTPParams()) )
 {
  unset($params["simpleSearch"]);
  if (array_key_exists("searchText",$params)) {
   $params["id"]=$params["searchText"];
   unset($params["searchText"]);
  }
  //set search parameters
  $search->setParams($params);
 }

 //get searched items
 $items=&$search->searchItems();
 $itemsCount=$search->itemsCount();
 //save search params to session
 $search->saveToSession();

 //page navigator
 $pgNav=new PageNav($pageNo,$itemsCount);
 $pgNav->MainUrl=$devUrlEnum->searchResult;

 //left block
 $this->view->leftBlocks = array(LeftBlock_SimpleSearch::getDeviceSearch());

 //number of found devices
 $this->view->foundDevices=$itemsCount;
 //devices in the current page
 $this->view->devices=$items;
 //device url enumerator for a view
 $this->view->devUrlEnum=$devUrlEnum;
 //add page navigator XHTML code for a view
 $this->view->pageNav=$pgNav->getNavXhtmlContent();
}

//==============================================================================
/**
 * Action handler, which returns a picture with the e1 summary for the current
 * device.
 * @return void
 * @see tsSumImgAction()
 */
public function e1SumImgAction()
{
 //object from the parameters
 $device=new CiscoDevice();
 $device->setDataById($this->_getParam("device"));

 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 $this->_setParam("Img",$device->SumPictures->getPathE1Pic());
 return;
}

//==============================================================================
/**
 * Action handler, which returns a picture with the timeslot summary for the current
 * device.
 * Sets Img param in the response object as the file system path for the image.
 * This image is then returned as stream in the controller plugin {@link SiteTemplatePlugin}.
 * @return void
 */
public function tsSumImgAction()
{
 //object from the parameters
 $device=new CiscoDevice();
 $device->setDataById($this->_getParam("device"));

 // Don't auto render
 $this->_helper->viewRenderer->setNoRender();
 $this->_setParam("Img",$device->SumPictures->getPathTimeslotPic());
 return;
}

#END CLASS
}
