<?php
/**
 * Implementation of the pagination support for the long tables or item lists.
 * 
 * @package Library
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Class for the page navigation support in the long lists of items.
 *
 * It can be used as view helper, has functions for a printing the XHTML code for
 * the page navigator for the list of items.
 *
 * @package Library
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2005-2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class PageNav
{
 /** current item offset
  * @var int
  */
 private $_current;

 /** number of all items
  * @var int
  */
 private $_allCount;

 /** default items per page
  */
 const PER_PAGE_COUNT = 15;

 /** items per page
  * @var int
  */
 private $_perPageCount;

 /** the number of all pages
  * @var int
  */
 private $_pgCount;

 /** current page number
  * @var int
  */
 private $_currentPageNum;

 /** how many pages as url links can be printed in page number line
  * @var int
  */
 private $_maxPageRend=10;

 /** URL without "page navigation" parameters
  *
  * after this url the page navigation URL parametr will be added
  * e.g.: http:/server.net/index.php?m=DeviceModule&a=viewal
  * @var string
  */
 private $_mainScriptUrl;

 /** default URL parameter for current page offset
  */
 const NAVPARAM = "page";

 /** URL parameter for current page offset
  * @var string
  */
 private $_offsetParam=self::NAVPARAM;

 /** print an offset page number in the url
  *
  * in default mode, page offset is printed
  * @var bool
  */
 private $_printItemOffset=false;

 /** content for the XHTML navigation code
  * @var string
  */
 private $_xhtmlNavContent;

//==============================================================================
/**
 * Creates a page navigator instance.
 *
 * @param int $currentPage current page offset, first page has number 1, but
 *  internally is counted from 0
 * @param int $allCount all items count
 * @param int $perPageCount items per page
 */
public function __construct($currentPage,$allCount,$perPageCount=self::PER_PAGE_COUNT)
{
 if ($currentPage>0) $this->_current=($currentPage-1)*$perPageCount;
 else $this->_current=0;
 $this->_allCount=$allCount;
 $this->_perPageCount=$perPageCount;

 //number of all pages
 $this->_pgCount=floor($this->_allCount/$this->_perPageCount);
 if ($this->_allCount % $this->_perPageCount) {
  $this->_pgCount++;
 }

 //current page
 $this->_currentPageNum=floor($this->_current/$this->_perPageCount);
 
 //reset content
 $this->_xhtmlNavContent="";
}

//==============================================================================
/**
 * Overloaded property setter.
 *
 * <b>Defined properties</b>
 * <ul>
 *   <li>Mainurl - get/set - {@link $_id}</li>
 *   <li>string IpAddr - get/set - {@link $_ipAddr}</li>
 * </ul>
 *
 * @throws InvalidArgumentException if $nonmVar parameter is the undefined property
 * or $val has a bad format or a type
 */
public function __set($nonmVar,$val)
{
 if ($nonmVar=="MainUrl") $this->_mainScriptUrl=$val;
 elseif ($nonmVar=="OffsetParam") $this->_offsetParam=$val;
 else {
  throw new InvalidArgumentException ("try to set undefined property $nonmVar or bad value "
                      . "for this property in the class ".get_class($this)
                      . (empty($val) ? "" : ("\n value dump: ".$val))
                     );
 }
}


//==============================================================================
/**
 * Gets current page number, this is $_currentPageNum+1, because
 * $_currentPageNum begins internally from 0.
 * @return int
 */
public function getCurrentPageNo()
{
 return $this->_currentPageNum+1;
}

//==============================================================================
/**
 * Gets items per page.
 * @return int
 */
public function getPerPageItems()
{
 return $this->_perPageCount;
}

//==============================================================================
/**
 * Enables the item offset printing.s
 */
public function setPrintItemOffset()
{
 $this->_printItemOffset=true;
}

//==============================================================================
/** Gets a page number for the item offset.
 * @param $offset int page offset
 * @return float
 */
private function _getPageNum($offset)
{
 return floor($offset/$this->_perPageCount)+1;
}

//==============================================================================
/**
 * Fills the XHTML content for the page navigator to the {@link $_xhtmlNavContent}.
 */
public function buildNavContent()
{
 if ($this->_pgCount <=1) return;
 $this->_renderFirst();
 $this->_renderPrevious();
 $this->_renderNmbLine();
 $this->_renderNext();
 $this->_renderLast();
}

//==============================================================================
/**
 * Get the XHTML content for the navigator, build the content if it's necessary.
 * @return string
 */
public function getNavXhtmlContent()
{
 if (empty($this->_xhtmlNavContent)) $this->buildNavContent();
 return $this->_xhtmlNavContent;
}

//==============================================================================
/**
 * Renders first part of the page navigator to the {@link $_xhtmlNavContent}.
 */
private function _renderFirst()
{
 if ($this->_currentPageNum==0) {
  $this->_xhtmlNavContent.="|&lt; ";
 } else {
  $url=$this->_makeUrl(0);
  $this->_xhtmlNavContent.="<a href=\"$url\" title=\"First page\">|&lt;</a> ";
 }
}

//==============================================================================
/**
 * Renders "previous" part of the page navigator to the {@link $_xhtmlNavContent}.
 */
private function _renderPrevious()
{
 if ($this->_currentPageNum==0) {
  $this->_xhtmlNavContent.= "&lt; ";
 } else {
  $previousOffset=($this->_currentPageNum-1)*$this->_perPageCount;
  $url=$this->_makeUrl($previousOffset);
  $this->_xhtmlNavContent.="<a href=\"$url\" title=\"Previuos page\">&lt;</a> \n";
 }
}

//==============================================================================
/**
 * Makes url for the page offset.
 * @param $offset page offset
 * @return string
 */
private function _makeUrl($offset)
{
 //navigation part in url
 $pageNum=$this->_getPageNum($offset);
 $navPart=$this->_offsetParam."=".$pageNum;
 if ($this->_printItemOffset) {
  $navPart=$this->_offsetParam."=".$offset;
 }

 if (preg_match("/\?/",$this->_mainScriptUrl)) {
  $url=$this->_mainScriptUrl."&amp;".$navPart;
 } else {
  $url=$this->_mainScriptUrl."?".$navPart;
 }

 return $url;
}

//==============================================================================
/**
 * Renders the line with page numbers to the {@link $_xhtmlNavContent}.
 */
private function _renderNmbLine()
{
 if (!$this->_pgCount>1) return;
 //echo "currentpg_n: $this->currentpg_n<br>";
 //get "first page" number - $startpn
 if ($this->_currentPageNum-$this->_maxPageRend+2 > 0) {
  $startPn=$this->_currentPageNum-($this->_maxPageRend/2);
 } else {
  $startPn=0;
 }
 //echo "startpn $startpn<br>";
 //render page bar
 for($i=$startPn;$i<($startPn+$this->_maxPageRend);$i++) {
  //page count reached, stop render and return
  if ($this->_pgCount==$i) return;
  $pgnmb=$i+1;
  if ($i==$this->_currentPageNum) {
   $this->_xhtmlNavContent.="<span style=\"font-weight: bold\">$pgnmb</span> \n";
   continue;
  }
  $offset=$i*$this->_perPageCount;
  $url=$this->_makeurl($offset);
  $this->_xhtmlNavContent.="<a href=\"$url\" title=\"Page no. $pgnmb\">$pgnmb</a> \n";
 }
}

//==============================================================================
/**
 * Renders the "next" part of the page navigator to the {@link $_xhtmlNavContent}.
 */
private function _renderNext()
{
 if ($this->_currentPageNum==$this->_pgCount-1) {
  $this->_xhtmlNavContent.="&gt; ";
 } else {
  $nextOffset=($this->_currentPageNum+1)*$this->_perPageCount;
  $url=$this->_makeUrl($nextOffset);
  $this->_xhtmlNavContent.= "<a href=\"$url\" title=\"Next page\">&gt;</a> ";
 }
}

//==============================================================================
/**
 * Renders the last part of the page navigator to the {@link $_xhtmlNavContent}.
 */
private function _renderLast()
{
 if ($this->_currentPageNum==$this->_pgCount-1) {
  $this->_xhtmlNavContent.="&gt;| ";
 } else {
  $lastOffset=($this->_pgCount-1)*$this->_perPageCount;
  $url=$this->_makeurl($lastOffset);
  $this->_xhtmlNavContent.="<a href=\"$url\" title=\"Last page\">&gt;|</a> \n";
 }
}

#END CLASS
}
