<?php
/**
* Implementation of the cisco device class
*
* @package ndevinfo
* @author Miroslav Kubelik (koubel@volny.cz)
* @copyright Copyright (c) 2007 Miroslav Kubelik
* @license http://opensource.org/licenses/gpl-license.php GNU GPL
*/

/**
* Class for the cisco device - Business (domain) class
*
* @package ndevinfo
* @author Miroslav Kubelik (koubel@volny.cz)
* @copyright Copyright (c) 2007 Miroslav Kubelik
* @license http://opensource.org/licenses/gpl-license.php GNU GPL
*/
class CiscoDevice extends Model implements IComparableEqual
{
 /**
  * device ID e.g. router1
  * @var string
  */
 private $_id;

 /**
  * device ip address e.g. router1.ispName.net
  * @var string
  */
 private $_ipAddr;

 /**
  * A date when a device report has been created.
  * @var DateTime
  */
 private $_reportCreateDate;

 /**
  * object with a summary statistics
  * @var SyncSum
  */
 private $_summary;

 /**
  * Summary pictures
  * @var SumPics
  */
 private $_sumPics;

//==============================================================================
public function __construct()
{
 $this->_dao = new CiscoDeviceXMLDao();
}

//==============================================================================
/** inherited property getter
* @param string $property property identifier
*
* <b>Defined properties</b>
* <ul>
*   <li>string Id - get/set - {@link $_id}</li>
*   <li>string IpAddr - get/set - {@link $_ipAddr}</li>
*   <li>DateTime CreatedDT - get - {@link $_reportCreateDate}</li>
*   <li>SyncSum Summary - get - {@link $_summary}</li>
*   <li>SumPics SumPictures - get - {@link $_sumPics}</li>
* </ul>
*
* @throws Exception if $property parameter is an undefined property identifier
* @return mixed $property value
*/
public function __get($property)
{
 switch ($property) {
  case "Id":
   return $this->_id;
   break;
  case "IpAddr":
   return $this->_ipAddr;
   break;
  case "CreatedDT":
   return $this->_reportCreateDate;
   break;
  case "Summary":
   return $this->_getSummary();
   break;
  case "SumPictures":
   return $this->_getSumPics();
   break;
  default:
   return parent::__get($property);
 	 break;
 }
}

//==============================================================================
/** inherited property setter
*
* For the properties definition, see {@link __get()}.
*
* @param string $property property identifier
* @param mixed $val property value
*
* @throws InvalidArgumentException if $property parameter is the undefined property identifier
*  or if $val is bad value for $property
*/
public function __set($property,$val)
{
 switch ($property) {
  case "Id":
   $this->_id=$val;
   break;
  case "IpAddr":
   $this->_ipAddr=$val;
 	 break;
  default:
   parent::__set($property);
 	 break;
 }
}

//==============================================================================
/**
 * Sets properties from the transfer object
 * @param CiscoDeviceTO $deviceTO transfer object
 */
private function _setFromTO(CiscoDeviceTO $deviceTO)
{
 $this->_id=$deviceTO->id;
 $this->_ipAddr=$deviceTO->ipAddr;
 $this->_reportCreateDate=$deviceTO->reportCreateDate;
}

//==============================================================================
/**
 * Sets properties for the device, which is identified by device Id
 * @param string $id device id, e.g. router1
 */
public function setDataById($id)
{
 $this->_setFromTO( $this->_dao->getDeviceById($id) );
}

//==============================================================================
/**
 * Gets summary for the device. Uses a lazy loadind for the $_summary
 * member variable.
 * @return SyncSum
 */
private function _getSummary()
{
 if (is_null($this->_summary)) {
  $this->_summary=new SyncSum();
  $this->_summary->setDaoBySummaryXml(CiscoDeviceXMLDao::getSummaryXML($this->_dao->getDeviceXML($this->_id)));
 }
 return $this->_summary;
}

//==============================================================================
/**
 * Gets summary pictures. Uses a lazy loadind for the $_sumPics member variable.
 * @return SumPics
 */
private function _getSumPics()
{
 if (is_null($this->_sumPics)) {
  $this->_sumPics=new SumPics($this->_dao->getMediaDir(),$this->_id);
 }
 return $this->_sumPics;
}

//==============================================================================
/**
 * Get all devices, returns an array of domain or transfer objects. Has the
 * pagination support.
 *
 * @param bool $returnTO if false(default) array of transfer objets is returned,
 *  if true, array of domain objects is returned
 * @param int $pageNo see {@link CiscoDeviceXMLDao::getAllDevices()}
 * @param int $perPageItems see {@link CiscoDeviceXMLDao::getAllDevices()}
 * @return array array of the CiscoDevice or CiscoDeviceTO objects
 */
public function &getAllDevices($pageNo=null,$perPageItems=null,$returnTO=false)
{
 $retArr=array();

 $allDevs=&$this->_dao->getAllDevices($pageNo,$perPageItems);
 if(count($allDevs)==0) return $retArr;

 if ($returnTO) return $allDevs;

 foreach($allDevs as $devTO) {
  $item=new self;
  $item->_setFromTO($devTO);
  $retArr[]=$item;
 }

 return $retArr;
}

//==============================================================================
/**
 * Get all devices count.
 * @return int
 */
public function getAllDevicesCount()
{
 return $this->_dao->getDevicesCount();
}

//==============================================================================
/**
 * Get E1 controller count for the current device.
 * @return int
 */
public function getAllE1ControllersCount()
{
 return $this->_dao->getAllE1ControllersCount($this->_id);
}

//==============================================================================
/**
 * Get all E1 controllers for the current device.
 * @return array array of the E1Contr objects
 */
public function &getAllE1Controllers()
{
 $contr=new E1Contr();
 $contr->setDaoByControllersXML(CiscoDeviceXMLDao::getE1ControllersXML($this->_dao->getDeviceXML($this->_id)));
 return $contr->getAllControllers();
}

//==============================================================================
/**
 * Get E3 controller count for the current device.
 * @return int
 */
public function getAllE3ControllersCount()
{
 return $this->_dao->getAllE3ControllersCount($this->_id);
}

//==============================================================================
/**
 * Get all E3 controllers for the current device.
 * @return array array of the E3Contr objects
 */
public function &getAllE3Controllers()
{
 $contr=new E3Contr();
 $contr->setDaoByControllersXML(CiscoDeviceXMLDao::getE3ControllersXML($this->_dao->getDeviceXML($this->_id)));
 return $contr->getAllControllers();
}

//==============================================================================
/**
 * Get Sonet controller count for the current device.
 * @return int
 */
public function getAllSonetControllersCount()
{
 return $this->_dao->getAllSonetControllersCount($this->_id);
}

//==============================================================================
/**
 * Get all Sonet controllers for the current device.
 * @return array array of the SonetContr objects
 */
public function &getAllSonetControllers()
{
 $contr=new SonetContr();
 $contr->setDaoByControllersXML(CiscoDeviceXMLDao::getSonetControllersXML($this->_dao->getDeviceXML($this->_id)));
 return $contr->getAllControllers();
}

//==============================================================================
/**
 * Checks, if parametr object is equal with THIS object instance
 * @return bool
 */
public function isEqual($valRef)
{
 if (!($valRef instanceof self)) return false;
 return ($valRef->_id==$this->_id && $valRef->_ipAddr==$this->_ipAddr);
}

//==============================================================================
/**
 * The comparsion function, which compare device by summaries, in the concrete its
 * whole summary part. It is the callback comparsion function for usort for
 * the array with CiscoDevice objects.
 * @param CiscoDevice $a first object
 * @param CiscoDevice $b second object
 * @return int, see http://www.php.net/usort
 */
public static function compareBySummary(CiscoDevice $a,CiscoDevice $b)
{
 $aE1SumFree=$a->_getSummary()->WholeSyncSummary->E1sSum->Free;
 $bE1SumFree=$b->_getSummary()->WholeSyncSummary->E1sSum->Free;

 //compare E1 summary
 if ($aE1SumFree == $bE1SumFree) {
  $aTsSumFree=$a->_getSummary()->WholeSyncSummary->TimeslotsSum->Free;
  $bTsSumFree=$b->_getSummary()->WholeSyncSummary->TimeslotsSum->Free;
  if ($aTsSumFree == $bTsSumFree) return 0;
  return ($aTsSumFree > $bTsSumFree) ? +1 :-1;
 }
 return ($aE1SumFree > $bE1SumFree) ? +1 : -1;
}

#END CLASS
}
