<?php
/**
 * Implementation of the cisco device XML dao class.
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * cisco device XML Dao class
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class CiscoDeviceXMLDao
{
 /**
  * Data source object, is used as source for the XML reports configuration and
  * for separated XML device reports also.
  * @var XMLDataSource object
  */
 private static $_dataSource;

//==============================================================================
/**
 * Inits data source.
 */
public function __construct()
{
 self::$_dataSource = XMLDataSource::getInstance();
}

//==============================================================================
/**
 * Get all XML device sections as XML element from the data source
 * @return SimpleXMLElement object
 */
private static function _getDevicesXML()
{
 return self::getDevicesXML( self::$_dataSource->getDevicesConfXML() );
}

//==============================================================================
/**
 * Get all XML device sections as XML element. This function is needed for
 * example XMLDataSource::_checkReports(), si it must be public, because PHP
 * doesn't support fried classes.
 *
 * @param SimpleXMLElement $config the device configuration XML
 * @return SimpleXMLElement object
 */
public static function getDevicesXML(SimpleXMLElement $config)
{
 return $config->devices->device;
}

//==============================================================================
/**
 * Gets media directory. It could be as static, but for encapsulation the dynamic
 * calling better, because this method can be called after $this is created.
 * @see XMLDataSource::getMediaDir()
 */
public function getMediaDir()
{
 return self::$_dataSource->getMediaDir();
}

//==============================================================================
public function getDeviceXML($deviceId)
{
 return self::$_dataSource->getDeviceReportById($deviceId);
}

//==============================================================================
/**
 * Get all device count.
 * @return int
 */
public function getDevicesCount()
{
 return count( self::_getDevicesXML() );
}

//==============================================================================
/**
 * Gets transfer object for a device by the device node Id.
 * @param string $nodeId the device id
 * @return CiscoDeviceTO object
 */
public function getDeviceById($nodeId)
{
 foreach(self::_getDevicesXML() as $devNode) {
  if ((string)$devNode["id"] == $nodeId) {
   $devTO=self::_getTO($devNode);
   //device XML report
   $deviceXMLReport=$this->getDeviceXML($devTO->id);
   self::_setTOfromReportXML($deviceXMLReport,$devTO);
   return $devTO;
  }
 }
 throw new Exception("XML node for the device ". $nodeId . " doesn't exist");
}

//==============================================================================
/**
 * Get all devices, a pagination is supported.
 * @param int $pageNo current page number, first page has number 1
 * @param int $perPageItems number of items per page
 * @return array array of the CiscoDeviceTO objects
 */
public function &getAllDevices($pageNo=null,$perPageItems=null)
{
 $usePages=false;
 if (is_numeric($pageNo) && is_numeric($perPageItems)) {
  $usePages=true;
  if ($pageNo<1) throw new RangeException("page number must be greater then 0");
  $pageNo--;
  $firstItemIndx=$pageNo*$perPageItems;
  $lastItemIndx=$firstItemIndx+$perPageItems;
 }

 $retArr=array();
 $allItems = self::_getDevicesXML();
 $itemCnt=0;
 foreach($allItems as $devNode) {
  if ($usePages) {
   $itemCnt++;
   if ($itemCnt>$firstItemIndx) {
    if ($itemCnt<=$lastItemIndx) {
     $retArr[]=self::_getTO($devNode);
     continue;
    }
    break;
   }
   continue;
  }
  $retArr[]=self::_getTO($devNode);
 }

 //array slice is not good for long item lists
 //if ($usePages) $retArr=array_slice($retArr,$pageNo*$perPageItems,$perPageItems);

 return $retArr;
}

//==============================================================================
/**
 * Gets all XML E1 controllers elements from the XML device element.
 * @param SimpleXMLElement $deviceXML the XML device element.
 * @return SimpleXMLElement
 */
public static function getE1ControllersXML(SimpleXMLElement $deviceXML)
{
 return $deviceXML->router->E1controllers->controller;
}

//==============================================================================
/**
 * Gets all E1 controllers count for the device.
 * @param int $deviceId device Id
 * @return int
 */
public function getAllE1ControllersCount($deviceId)
{
 return count( self::getE1ControllersXML($this->getDeviceXML($deviceId)) );
}

//==============================================================================
/**
 * Gets all XML E3 controllers elements from the XML device element.
 * @param SimpleXMLElement $deviceXML the XML device element.
 * @return SimpleXMLElement
 */
public static function getE3ControllersXML(SimpleXMLElement $deviceXML)
{
 return $deviceXML->router->E3controllers->E3controller;
}

//==============================================================================
/**
 * Gets all E3 controllers count for the device.
 * @param int $deviceId device Id
 * @return int
 */
public function getAllE3ControllersCount($deviceId)
{
 return count( self::getE3ControllersXML($this->getDeviceXML($deviceId)) );
}

//==============================================================================
/**
 * Gets all XML Sonet controllers elements from the XML device element.
 * @param SimpleXMLElement $deviceXML the XML device element.
 * @return SimpleXMLElement
 */
public static function getSonetControllersXML(SimpleXMLElement $deviceXML)
{
 return $deviceXML->router->Sonetcontrollers->Sonetcontroller;
}

//==============================================================================
/**
 * Gets all Sonet controllers count for the device.
 * @param int $deviceId device Id
 * @return int
 */
public function getAllSonetControllersCount($deviceId)
{
 return count( self::getSonetControllersXML($this->getDeviceXML($deviceId)) ) ;
}

//==============================================================================
/**
 * Gets a XML summary element from the XML device element.
 * @param SimpleXMLElement $deviceXML the XML device element.
 * @return SimpleXMLElement
 */
public static function getSummaryXML(SimpleXMLElement $deviceXML)
{
 return $deviceXML->router->summary;
}

//==============================================================================
/**
 * Sets transfer object properties from the device XML element from the
 * configuration XML file.
 * @param SimpleXMLElement $elem device element from the XML configuration
 * @param CiscoDeviceTO $devTO a transfer object, its properties will be set
 */
private static function _setTOfromConfigXML(SimpleXMLElement $elem,CiscoDeviceTO $devTO)
{
 foreach ($elem->attributes() as $attrName => $value) {
  switch ((string)$attrName) {
   case "id":
    $devTO->id=(string)$value;
    break;
   case "ipaddr":
    $devTO->ipAddr=(string)$value;
    break;
   default:
 	 break;
  }
 }
}

//==============================================================================
/**
 * Sets transfer object properties from the device report XML element from the
 * XML report for one device.
 * @param SimpleXMLElement $elem device report XML element
 * @param CiscoDeviceTO $devTO a transfer object, its properties will be set
 */
private static function _setTOfromReportXML(SimpleXMLElement $elem,CiscoDeviceTO $devTO)
{
 $strDT=(string)$elem->report["created"];
 $devTO->reportCreateDate=date_create($strDT);
}

//==============================================================================
/**
 * build a corresponding Transfer Object from the device XML element from
 * the configuration XML file.
 * @param SimpleXMLElement $devElement device element from the XML configuration
 * @return CiscoDeviceTO
 */
private static function _getTO(SimpleXMLElement $devElement)
{
 $ourTO=new CiscoDeviceTO();
 self::_setTOfromConfigXML($devElement,$ourTO);
 return $ourTO;
}


#END CLASS
}
