<?php
/**
* Implementation of the group of devices
*
* @package ndevinfo
* @author Miroslav Kubelik (koubel@volny.cz)
* @copyright Copyright (c) 2007 Miroslav Kubelik
* @license http://opensource.org/licenses/gpl-license.php GNU GPL
*/

/**
* Class for the device group - Business (domain) class
*
* @package ndevinfo
* @author Miroslav Kubelik (koubel@volny.cz)
* @copyright Copyright (c) 2007 Miroslav Kubelik
* @license http://opensource.org/licenses/gpl-license.php GNU GPL
*/
class DeviceGroup extends Model implements IComparableEqual
{
 /**
  * group name
  * @var string
  */
 private $_name;

 /**
  * the report name for the XML group report
  * @var string
  */
 private $_reportName;

 /**
  * A date when a report has been created.
  * @var DateTime
  */
 private $_reportCreateDate;
 
 /**
  * object with a summary statistics
  * @var SyncSum
  */
 private $_summary;

 /**
  * Summary pictures
  * @var SumPics
  */
 private $_sumPics;

//==============================================================================
/**
 * Initilizes instance, creates Dao.
 */
public function __construct()
{
 $this->_dao = new DeviceGroupXMLDao();
}

//==============================================================================
/**
 * Inherited property getter.
 * @param string $property property identifier
 *
 * <b>Defined properties</b>
 * <ul>
 *  <li>string Name - get/set - {@link $_name}</li>
 *  <li>string ReportName - get/set - {@link $_reportName}</li>
 *  <li>DateTime CreatedDT - get - {@link $_reportCreateDate}</li>
 *  <li>SyncSum Summary - get - {@link $_summary}</li>
 *  <li>SumPics SumPictures - get - {@link $_sumPics}</li>
 * </ul>
 *
 * @throws Exception if $property parameter is an undefined property identifier
 * @return mixed $property value
*/
public function __get($property)
{
 switch ($property) {
  case "Name":
   return $this->_name;
   break;
  case "ReportName":
   return $this->_reportName;
   break;
  case "CreatedDT":
   return $this->_reportCreateDate;
   break;
  case "Summary":
   return $this->_getSummary();
   break;
  case "SumPictures":
   return $this->_getSumPics();
   break;
  default:
   return parent::__get($property);
 	 break;
 }
}

//==============================================================================
/**
* Inherited property setter
*
* For the properties definition, see {@link __get()}.
*
* @param string $property property identifier
* @param mixed $val property value
*
* @throws InvalidArgumentException if $property parameter is the undefined property identifier
*  or if $val is bad value for $property
*/
public function __set($property,$val)
{
 switch ($property) {
  case "Name":
   $this->_id=$val;
   break;
  case "ReportName":
   $this->_reportName=$val;
   break;
  default:
   parent::__set($property);
 	 break;
 }
}

//==============================================================================
/**
 * Sets properties from the transfer object
 * @param DeviceGroupTO $groupTO transfer object
 */
private function _setFromTO(DeviceGroupTO $groupTO)
{
 $this->_name=$groupTO->name;
 $this->_reportName=$groupTO->reportName;
 $this->_reportCreateDate=$groupTO->reportCreateDate;
}

//==============================================================================
/**
 * Get all groups, returns an array of domain or transfer objects. Has the
 * pagination support.
 *
 * @param bool $returnTO if false(default) array of transfer objets is returned,
 *  if true, array of domain objects is returned
 * @param int $pageNo see {@link DeviceGroupXMLDao::getAllGroups()}
 * @param int $perPageItems see {@link DeviceGroupXMLDao::::getAllGroups()}
 * @return array array of the DeviceGroup or DeviceGroupTO objects
 */
public function &getAllGroups($pageNo=null,$perPageItems=null,$returnTO=false)
{
 $retArr=array();

 $allGroups=&$this->_dao->getAllGroups($pageNo,$perPageItems);
 if(count($allGroups)==0) return $retArr;

 if ($returnTO) return $allGroups;

 foreach($allGroups as $groupTO) {
  $item=new self;
  $item->_setFromTO($groupTO);
  $retArr[]=$item;
 }

 return $retArr;
}

//==============================================================================
/**
 * Get all groups count.
 * @return int
 */
public function getAllGroupsCount()
{
 return $this->_dao->getGroupsCount();
}

//==============================================================================
/**
 * Sets properties for the group, which is identified by group name
 * @param string $name group name
 */
public function setDataByName($name)
{
 $this->_setFromTO( $this->_dao->getGroupByName($name) );
}

//==============================================================================
/**
 * Checks, if parametr object is equal with THIS object instance
 * @return bool
 */
public function isEqual($valRef)
{
 if (!($valRef instanceof self)) return false;
 return ($valRef->_name==$this->_name
         && $valRef->_reportName==$this->_reportName);
}

//==============================================================================
/**
 * Get members count.
 * @return int
 */
public function getMembersCount()
{
 return $this->_dao->getGroupMembersCount($this->_name);
}

//==============================================================================
/**
 * Get all members.
 * @return array array of CiscoDevice
 */
public function &getMembers()
{
 $memberIds=&$this->_dao->getGroupMemberIds($this->_name);

 //returned array with members
 $devArr=array();

 if (!count($memberIds)) return $devArr;

 foreach($memberIds as $devId) {
  $device=new CiscoDevice();
  $device->setDataById($devId);
  $devArr[]=$device;
 }

 //sort array
 usort($devArr,array("CiscoDevice","compareBySummary"));
 return $devArr;
}

//==============================================================================
/**
 * Gets summary for the group. Uses a lazy loadind for the $_summary
 * member variable.
 */
private function _getSummary()
{
 if (is_null($this->_summary)) {
  $this->_summary=new SyncSum();
  $this->_summary->setDaoBySummaryXml(DeviceGroupXMLDao::getSummaryXML($this->_dao->getGroupXML($this->_reportName)));
 }
 return $this->_summary;
}

//==============================================================================
/**
 * Gets summary pictures. Uses a lazy loadind for the $_sumPics member variable.
 * @return SumPics
 */
private function _getSumPics()
{
 if (is_null($this->_sumPics)) {
  $this->_sumPics=new SumPics($this->_dao->getMediaDir(),$this->_reportName);
 }
 return $this->_sumPics;
}

#END CLASS
}

