<?php
/**
 * Implementation of the device group XML dao class.
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Device group XML Dao class.
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class DeviceGroupXMLDao
{
 /**
  * Data source object, is used as source for the XML reports configuration and
  * for separated XML group reports also.
  * @var XMLDataSource object
  */
 private static $_dataSource;

//==============================================================================
/**
 * Inits data source.
 */
public function __construct()
{
 self::$_dataSource = XMLDataSource::getInstance();
}

//==============================================================================
/**
 * Get all XML group sections as XML element from the device and group XML
 * configuration file.
 * @return SimpleXMLElement object
 */
private static function _getAllGroupsConfXML()
{
 return self::$_dataSource->getDevicesConfXML()->groups->group;
}

//==============================================================================
private function _getGroupConfXML($name)
{
 foreach(self::_getAllGroupsConfXML() as $groupNode) {
  if ((string)$groupNode["name"] == $name) {
   return $groupNode;
  }
 }

 throw new Exception("XML node for the group ". $groupNode . " doesn't exist");
}

//==============================================================================
/**
 * Get all groups count.
 * @return int
 */
public function getGroupsCount()
{
 return count( self::_getAllGroupsConfXML() );
}

//==============================================================================
public function getGroupXML($reportName)
{
 return self::$_dataSource->getGroupReportByReportName($reportName);
}

//==============================================================================
/**
 * Gets a XML summary element from the XML group report element.
 * @param SimpleXMLElement $groupXMLReport the XML group report element.
 * @return SimpleXMLElement
 */
public static function getSummaryXML(SimpleXMLElement $groupXMLReport)
{
 return $groupXMLReport->summary;
}

//==============================================================================
/**
 * Gets media directory.
 * @see XMLDataSource::getMediaDir()
 */
public function getMediaDir()
{
 return self::$_dataSource->getMediaDir();
}

//==============================================================================
/**
 * Sets transfer object properties from the group XML element from the
 * configuration XML file.
 * @param SimpleXMLElement $elem group element from the XML configuration
 * @param DeviceGroupTO $groupTO a transfer object, its properties will be set
 */
private static function _setTOfromConfigXML(SimpleXMLElement $elem,DeviceGroupTO $groupTO)
{
 foreach ($elem->attributes() as $attrName => $value) {
  switch ((string)$attrName) {
   case "name":
    $groupTO->name=(string)$value;
    break;
   default:
 	 break;
  }
 }
 
 $groupTO->reportName=(string)$elem->report;
}

//==============================================================================
/**
 * Sets transfer object properties from the group report XML element from the
 * XML report for one group.
 * @param SimpleXMLElement $elem group report XML element
 * @param DeviceGroupTO $groupTO a transfer object, its properties will be set
 */
private static function _setTOfromReportXML(SimpleXMLElement $elem,DeviceGroupTO $groupTO)
{
 $strDT=(string)$elem->report["created"];
 $groupTO->reportCreateDate=date_create($strDT);
}

//==============================================================================
/**
 * build a corresponding Transfer Object from the group XML element from
 * the configuration XML file.
 * @param SimpleXMLElement $groupElement group element from the XML configuration
 * @return DeviceGroupTO
 */
private static function _getTO(SimpleXMLElement $groupElement)
{
 $ourTO=new DeviceGroupTO();
 self::_setTOfromConfigXML($groupElement,$ourTO);
 return $ourTO;
}

//==============================================================================
/**
 * Gets transfer object for a group by the group name.
 * @param string $name the group name
 * @return DeviceGroupTO
 */
public function getGroupByName($name)
{
 $groupConfXML=self::_getGroupConfXML($name);
 $groupTO=self::_getTO($groupConfXML);
 //group XML report from a group report XML
 $groupXMLReport=$this->getGroupXML($groupTO->reportName);
 self::_setTOfromReportXML($groupXMLReport,$groupTO);
 return $groupTO;
}

//==============================================================================
/**
 * Get all groups, a pagination is supported.
 * @param int $pageNo current page number, first page has number 1
 * @param int $perPageItems number of items per page
 * @return array array of the DeviceGroupTO objects
 */
public function &getAllGroups($pageNo=null,$perPageItems=null)
{
 $usePages=false;
 if (is_numeric($pageNo) && is_numeric($perPageItems)) {
  $usePages=true;
  if ($pageNo<1) throw new RangeException("page number must be greater then 0");
  $pageNo--;
  $firstItemIndx=$pageNo*$perPageItems;
  $lastItemIndx=$firstItemIndx+$perPageItems;
 }

 $retArr=array();
 $allItems = self::_getAllGroupsConfXML();
 $itemCnt=0;
 foreach($allItems as $groupNode) {
  if ($usePages) {
   $itemCnt++;
   if ($itemCnt>$firstItemIndx) {
    if ($itemCnt<=$lastItemIndx) {
     $retArr[]=self::_getTO($groupNode);
     continue;
    }
    break;
   }
   continue;
  }
  $retArr[]=self::_getTO($groupNode);
 }

 //array slice is not good for long item lists
 //if ($usePages) $retArr=array_slice($retArr,$pageNo*$perPageItems,$perPageItems);

 return $retArr;
}

//==============================================================================
/**
 * Get group member names, it means device names.
 * @param string $name group name
 * @return array array of strings - device Ids
 */
public function &getGroupMemberIds($name)
{
 $membersXML=self::_getGroupConfXML($name)->member;

 $devIds=array();
 foreach($membersXML as $node) {
  $devIds[]=(string)$node;
 }
 
 return $devIds;
}

//==============================================================================
/**
 * Get group members count
 * @param string $name group name
 * @return int
 */
public function getGroupMembersCount($name)
{
 return count($membersXML=self::_getGroupConfXML($name)->member);
}

#END CLASS
}
