<?php
/**
 * An implementation of the general search object
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2006-2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Generic abstract search class
 *
 * Support for a simple and complex search, pagination.
 * This class will be inherited by device search, etc.
 *
 * This class can be used from CLI, be aware, that sessions cannot be used for the
 * CLI SAPI.
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2006-2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
abstract class Search
{
 /** a name for search session namespace */
 const SESSION_NAME = "ndevinfosearch";
 /** enable logging*/
 const ENABLE_LOG = false;

 /**
  * Dao class for the searched object
  * @var Zend_Db object
  */
 protected $_dao=null;

 /**
  * Parameters for a searching.
  * @var array
  */
 protected $_searchParams=array();

 /** Use pagination or not
  * default yes
  * @var bool
  */
 public $_paging=true;

 /**
 * current page number
 */
 protected $_pageNumber;

 /**
 * items per page
 */
 protected $_perPageItems;

 /**
  * session accessor for a searching
  * @var Zend_Session_Namespace
  */
 protected static $_searchSession;

 /**
  * logger object
  * @var Zend_Log
  */
 private static $_log;

 /**
  * Number of found items.
  * @var int
  */
 protected $_foundCount;

//==============================================================================
/**
 * Initialize Dao, set session name and load seach parameters from the session,
 * if necessary.
 *
 * @param object $dao Dao class for the searching
 */
protected function __construct($dao)
{
 $this->_dao=$dao;
 //get params from session
 //session must be started, it is OK, because session is started in the
 //factorySearch method
 $this->_searchParams =  isset(self::$_searchSession->searchParams) ? self::$_searchSession->searchParams : array();
}

//==============================================================================
/**
 * Sets paging.
 */
public function enablePaging()
{
 $this->_paging=true;
}

//==============================================================================
/**
 * Unset paging.
 */
public function disablePaging()
{
 $this->_paging=false;
}

//==============================================================================
/**
 * Create class logger
 */
public static function createLogger()
{
 $config = Zend_Registry::get("conf");
 $searchLogFile=$config->logdir.DIRECTORY_SEPARATOR."searchLog";
 $stream = @fopen($searchLogFile, 'w', false);
 if (!$stream) {
  throw new Exception("Failed to open a stream {$searchLogFile} for the search loggging.");
 }
 $writer =  new Zend_Log_Writer_Stream($stream);
 self::$_log = new Zend_Log($writer);
}

//==============================================================================
/**
 * Factory method for the Search object.
 *
 * @param object $dao Dao object
 * @param string $searchObjType name for the search class, which we want create. If is
 * empty, name is loaded from the session.
 * @return Search search instance reference
 */
public static function factorySearch($searchObjType="",$dao=null)
{
 self::_logMessage(__METHOD__.": searchObjType: $searchObjType");

 //starts session, if necessary, don't start session in the cli environment
 if (php_sapi_name() != 'cli')
  self::$_searchSession = new Zend_Session_Namespace(self::SESSION_NAME);

 $className = !empty($searchObjType) ? $searchObjType : "";
 if (empty($className)) {
  if (isset(self::$_searchSession->searchClassName)) {
   $className = self::$_searchSession->searchClassName;
  }
 }
                
 if (empty($className))
  throw new RuntimeException("Cannot create search object, empty className");

 switch ($className) {
  case "SearchDevice" :
   self::_logMessage("create Search_Device");
   return new Search_Device();
   break;
  case "SearchDevGroup" :
   self::_logMessage("create Search_DevGroup");
   return new Search_DevGroup();
   break;
  default:
   throw new Exception("Cannot create search object, unknow classname: $className");
 }
}

//==============================================================================
/**
 * Set search parameters. The setter for $_searchParams.
 * @param arrayref $arr array with search parameters
 */
public function setParams(&$arr)
{
 $this->_searchParams=$arr;
}

//==============================================================================
/**
 * Set pagination members.
 * @param int $pageNum current page number
 * @param int $itemsInPage items per page
 */
public function setPageParams($pageNum,$itemsInPage)
{
 $this->_pageNumber=$pageNum;
 $this->_perPageItems=$itemsInPage;
}

//==============================================================================
/**
 * Search items.
 * @return arrayref reference to array with found items
 */
abstract public function &searchItems();

//==============================================================================
/**
 * Get number of found items. Getter for the $_foundCount.
 * @return int
 * @throws RuntimeException if $_foundCount isn't initialized
 */
public function itemsCount()
{
 if (!isset($this->_foundCount))
  throw new RuntimeException("number of found items isn't set, you must call searchItems()
  method first");

 return $this->_foundCount;
}

//==============================================================================
/*
* Saves information about current searching into the session.
*/
public function saveToSession()
{
 self::_logMessage(get_class($this)." ".__METHOD__);
 self::$_searchSession->searchParams=$this->_searchParams;
 self::$_searchSession->searchClassName=get_class($this);
}

//==============================================================================
/*
* Clear information about current searching from the sessions.
*/
public static function delFromSession()
{
 self::_logMessage(get_class()." ".__METHOD__);
 /* we must create session namespace, because this static
    method can be called itself - without factory, which creates
    self::$_searchSession
 */
 if (php_sapi_name() != 'cli') {
  $sessionAccess = new Zend_Session_Namespace(self::SESSION_NAME);
  Zend_Session::namespaceUnset(self::SESSION_NAME);
 }
}

//==============================================================================
/*
* Send a message to log.
* @param string $msg message
*/
protected static function _logMessage($msg)
{
 if (self::ENABLE_LOG) {
  //create logger if necessary
  if (!self::$_log instanceof Zend_Log) self::createLogger();
  self::$_log->log($msg,Zend_log::DEBUG);
 }
}

#END CLASS
}
