<?php
/**
 * Implementation of the XML data source class
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Class encapsulates operation with XML files, which are application
 * data source. There are two main data source:
 * - application configuration
 * - reports
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class XMLDataSource
{
 /**
  * Singleton instance
  * @var XMLDataSource
  */
 private static $_instance = null;

 /**
  * directory with all reports
  * @var string
  */
 private $_reportDir;

 /**
  * directory with all media files (rrd databases, generated pictures)
  * @var string
  */
 private $_mediaDir;

 /**
  * object for the XML configuration for devices and groups
  * @var SimpleXMLElement object
  */
 private $_configuration;
 
//==============================================================================
/**
 * Singleton pattern implementation makes "new" unavailable
 * @return void
 */
private function __construct()
{
 $config=Zend_Registry::get("conf");
 $this->_setXMLConf($config->deviceXMLConfig);
}

//==============================================================================
/**
 * Singleton pattern implementation makes "clone" unavailable
 * @return void
 */
private function __clone()
{}

//==============================================================================
/**
 * Singleton pattern implementation
 * only one instance for a whole system
 * @return XMLDataSource
 */
public static function getInstance()
{
 if (null === self::$_instance) {
  self::$_instance = new self();
 }
 return self::$_instance;
}

//==============================================================================
/**
 * Reads the XML configuration file for devices, groups and reports and sets
 * class properties.
 * @param string $filePath a full path for the XML configuration file
 * @throws Exception when additional checks for reports fails
 */
private function _setXMLConf($filePath)
{
 if (file_exists($filePath)) {
  $this->_configuration = simplexml_load_file($filePath);
 }
 else
  throw new Exception("the xml file $filePath doesn't exist.");

 $this->_reportDir = (string)$this->_configuration->reportDir;
 $this->_mediaDir = (string)$this->_configuration->mediaDir;
 $this->_checkReports();
}

//==============================================================================
/**
 * Checks if all eports defined in the configuration exists.
 * @return Exception if some XML export doesn't exist.
 */
private function _checkReports()
{
 if (!file_exists($this->_reportDir)) throw new Exception("a directory $this->_reportDir doesn't exist.");

 $allDevs = CiscoDeviceXMLDao::getDevicesXML( $this->_configuration );
 foreach($allDevs as $devNode) {
  if (file_exists( $this->_reportDir . "/" . $devNode["id"] . ".xml" ))
   continue;
  else throw new Exception("report for a device ". $devNode["id"] . " doesn't exist");
 }
}

//==============================================================================
/**
 * Gets XML object with the configuration for devices and groups
 * @return SimpleXMLElement
 */
public function getDevicesConfXML()
{
 return $this->_configuration;
}

//==============================================================================
/**
 * Gets the media directory path. Accessor for {@link $_mediaDir}
 * @return string
 */
public function getMediaDir()
{
 return $this->_mediaDir;
}

//==============================================================================
/**
 * Get the device report selected by id as the XML object.
 * @param string $deviceId device indetifier, from this is derived XML file name.
 * @return SimpleXMLElement object
 */
public function getDeviceReportById($deviceId)
{
 $reportFile = $this->_reportDir."/".$deviceId.".xml";
 return simplexml_load_file($reportFile);
}

//==============================================================================
/**
 * Get the group report selected by name as the XML object.
 * @param string $reportName a name for the group report, from this is derived XML file name,
 *  be aware, that group name cannot be same as group report name.
 * @return SimpleXMLElement object
 */
public function getGroupReportByReportName($reportName)
{
 $reportFile = $this->_reportDir."/".$reportName.".xml";
 return simplexml_load_file($reportFile);
}

#END CLASS
}
