<?php
/**
 * Implementation of a helper which renders E1 timeslots in an E1 table.
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */

/**
 * Helper which renders E1 timeslots in an E1 table.
 *
 * @package ndevinfo
 * @author Miroslav Kubelik (koubel@volny.cz)
 * @copyright Copyright (c) 2007 Miroslav Kubelik
 * @license http://opensource.org/licenses/gpl-license.php GNU GPL
 */
class NdevInfo_View_Helper_E1TableTimeslots
{
//==============================================================================
/**
 * Generates <td> element for the E1 table row.
 * 
 * @param E1Contr $e1Controller E1 controller with our channel groups
 * @return string corresponding XHTML markup
 */
public function e1TableTimeslots(E1Contr $e1Controller)
{
 $xHtml="";

 //no channel groups, render as free E1 and return
 if ($e1Controller->getAllChannelGroupsCount()==0) {
  $xHtml="<td class=\"freee1\" colspan=\"32\" "
   . "title=\"Free E1 32 timeslots\">Free E1</td>";
  return $xHtml;
 }

 //channel group array
 $channelGroupArr=&$e1Controller->getAllChannelGroups();

 //unframed, print unframed and return return
 if ( count($channelGroupArr)==1 && $channelGroupArr[0]->isUnframed() ) {
  $chng=&$channelGroupArr[0];
  $title=self::getChanGrpTitle($chng);
  $xHtml="<td class=\"used\" colspan=\"32\" title=\"$title\">unframed</td>";
  return $xHtml;
 }

 //print 0 timeslot
 $xHtml="<td class=\"sync\" title=\"SYNC\">0</td>\n";

 //print channel groups in the 1-31 timeslot cycle
 $currentTS=1;
 $freeFlag=0;
 $freeTSCount=0;
 while ($currentTS<32) {
  $beginTS=$currentTS;
  //if (defined('DEBUG')) echolog("beginTS: $begints\n");
  $lastTS = self::getLastTS($beginTS,$channelGroupArr);
  //if (defined('DEBUG')) echolog("lastTS: $lastts\n");
  //timeslot is free
  if ( $lastTS == -1 ) {
   //previous TS was also free
   if ( $freeFlag == 1 ) { $freeTSCount++; }
   //first free TS
   else { $freeFlag = 1; $freeTSCount++; }
   //this is last E1 timeslot, print output
   if ( $currentTS == 31 ) {
    $xHtml.=self::renderFreeTS($beginTS - ( $freeTSCount - 1 ),$freeTSCount - 1);
   }
   $currentTS++;
   continue;
  }
  //timeslot is not free, in the $lastTS is last used timeslot (included last TS in whole E1)
  else {
   //print previous free TS if used
   if ( $freeFlag == 1 ) {
    //if (defined('DEBUG')) echolog("freetscnt $freetscnt\n");
    $xHtml.=self::renderFreeTS($beginTS - 1 - ( $freeTSCount - 1 ),$freeTSCount - 1);
    $freeTSCount=0;
    $freeFlag=0;
   }
   //find a channel group with a $beginTS timeslot
   $chanGrp=self::findChnGrpByFirstTS($beginTS,$channelGroupArr);
   $xHtml.=self::renderUsedTS($beginTS,$lastTS - $beginTS,$chanGrp);
   $currentTS = $lastTS + 1;
   continue;
  }
 }
 return $xHtml;
}

//=============================================================================
/**
 * Gets a last timeslot from the timeslot group in which is first timeslot.
 * Cycles through all channel-groups.
 * @param int $firstTS firts timeslot
 * @param arrayref $allE1chng array of ChannelGroup objects for one E1 controller
 * @return int the end timeslot number, -1 if firts timeslot isn't in any channel group
 */
private static function getLastTS($firstTS,&$allE1ChanGrp)
{
 //go through all channelgroups
 foreach ($allE1ChanGrp as $chanGrp){
  $tsRange = $chanGrp->TsRange;
  //one timeslot only
  if (preg_match("/^[0-9]+$/",$tsRange)) {
   if ( $firstTS == $tsRange ) return $tsRange;
   continue;
  }
  //cycle for timeslot separation ex: 1-16,25
  foreach (preg_split("/,/",$tsRange) as $groupTS) {
   //if (defined('DEBUG')) echolog("groupts:$groupts\n");
   //one timeslot only
   if (preg_match("/^[0-9]+$/",$groupTS)) {
    if ( $firstTS == $groupTS ) return $groupTS;
    continue;
   }
   list ($beginTS,$endTS) = preg_split("/-/",$groupTS);
   if ( $firstTS == $beginTS ) return $endTS;
  }
 }

 //$firstTS is not begin timeslot in any channelgroup
 return -1;
}

//=============================================================================
/**
 * Renders free timeslost group.
 * @param int $firstTS first free timeslot number
 * @param int $tsCounter a number f free timeslots
 * @return string xhtml markup for a free group of timeslots
 */
private static function renderFreeTS($firstTS,$tsCounter)
{
 $lastTS = $firstTS + $tsCounter;
 //if (defined('DEBUG')) echolog("printing free group: beginTS: $firstTS, lastTS $lastTS\n");
 //only one timeslot
 if ($tsCounter == 0 ) {
  return "<td class=\"free\" title=\"Free ".($tsCounter+1)." timeslot\">$firstTS</td>\n";
 }
 else {
  //more timeslosts
  return "<td class=\"free\" colspan=\"".($tsCounter+1)
   ."\" title=\"Free ".($tsCounter+1)." timeslots\">$firstTS-$lastTS</td>\n";
 }
}

//=============================================================================
/**
 * Renders an used timeslost group with a description
 * @param int $firstTS first timeslot number for an used group
 * @param int $tsCounter a number of used timeslots
 * @param ChannelGroup $chgrp corresponding channel group object
 * @return string xhtml markup for a free group of timeslots
 */
private static function renderUsedTS($firstTS,$tsCounter,ChannelGroup $chgrp)
{
 $title=self::getChanGrpTitle($chgrp);
 
 $lastTS = $firstTS + $tsCounter;

 //if (defined('DEBUG')) echolog("printing used group: beginTS: $firstTS, lastTS $lastTS channelgroup: {$chgrp['@']['id']}\n");

 $xHtml="<td class=\"used\"";

 //more timeslots -> colspan before tooltip!!
 if ( $tsCounter > 0 ) {
   $xHtml.=" colspan=\"".($tsCounter+1)."\"";
 }

 $xHtml.=" title=\"$title\">";

 //timeslots range text  e.g. 1, 20-24 ...
 //only one timeslot
 if ( $tsCounter == 0 )
  $tsRange=$firstTS;
 //more timeslosts
 else
  $tsRange="$firstTS-$lastTS";

 $xHtml.="$tsRange</td>\n";

 return $xHtml;
}

//=============================================================================
/**
 * Returns channel group object for specific first timeslot.
 * @param int $firstTS searched timeslot number
 * @param arrayref $allE1ChanGrp array of ChannelGroup objects for one E1 controller
 * @return ChannelGroup found channel group object with a firts timeslot $firstTS, null
 * if no corresponding channel group found.
 */
private static function findChnGrpByFirstTS($firstTS,&$allE1ChanGrp)
{
 //go through all channelgroups
 foreach ($allE1ChanGrp as $chanGrp){
  $tsRange = $chanGrp->TsRange;
  //one timeslot only
  if (preg_match("/^[0-9]+$/",$tsRange)) {
   if ( $firstTS == $tsRange ) return $chanGrp;
   continue;
  }
  //Cycle for timeslot separation ex: 1-16,25
  foreach (preg_split("/,/",$tsRange) as $groupTS) {
   //one timeslot only
   if (preg_match("/^[0-9]+$/",$groupTS)) {
    if ( $firstTS == $groupTS ) return $chanGrp;
    continue;
   }
   list ($beginTS,$endTS) = preg_split("/-/",$groupTS);
   if ( $firstTS == $beginTS ) return $chanGrp;
  }
 }

 //$firstTS is not begin TS in any channel-group
 return null;
}

//=============================================================================
/**
 * Returns title string for the channel group title attribute.
 * @param ChannelGroup $chanGrp channel group object
 * @return string title string
 */
private static function getChanGrpTitle(ChannelGroup $chanGrp)
{
 return "channel-group $chanGrp->Id -"
  ." IP Address: ".(strlen($chanGrp->IpAddr) ? $chanGrp->IpAddr : "undef")
  ." Description: ".(strlen($chanGrp->Description) ?  htmlspecialchars($chanGrp->Description) : "undef");
}

#END CLASS
}
